import pytest

from ipsurv.data_collector.pass_data_collector import PassDataCollector, PassRequester
from ipsurv.data_collector.basic_collectors import RdapCollector, DnsTxtCollector, IpInfoCollector, DnsReverseCollector
from ipsurv.data_collector.reactivity_collectors import ICMPCollector, TCPCollector, UDPCollector, HttpCollector
from ipsurv.data_collector.self_collector import SelfCollector
from ipsurv.requester.rdap import RdapRequester, CountryDetector
from ipsurv.requester.dns_resolver import DnsResolveRequester
from ipsurv.requester.ip_info import IpInfoRequester
from ipsurv.requester.server_reactivity import ServerReactivity
from ipsurv.requester.http import HttpRequester

from ipsurv.core.entity import Target, ValueData

import time
import re
import logging
import os


@pytest.fixture
def args(mocker):
    args = mocker.Mock()
    args.fixed_timeout = {'dns': 3, 'http': 3, 'reactivity': 3}
    args.fixed_format = '{ip}'
    args.fixed_format_params = ['ip']
    args.fixed_delimiter = ','
    args.alt_delimiter = ';'
    args.fixed_enclose = ''
    args.exhaustive = False

    return args


@pytest.fixture
def data(mocker):
    data = ValueData({})

    return data


@pytest.fixture
def requester(mocker):
    requester = mocker.Mock()

    return requester


class TestDataCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, requester, args):
        collector = PassDataCollector('TEST', requester, args)

        assert collector.get_name() == 'TEST'

        assert hasattr(collector, 'initialize')

        begin_time = time.time()
        time.sleep(0.5)
        v = collector._get_measure_time(begin_time)

        assert v < 2000 and v > 0

    def test_put(self, requester, args):
        collector = PassDataCollector('TEST', requester, args)

        data = ValueData({})
        response = {'a': 1, 'b': 2, 'c': 3}

        collector.put(data, response, 'a')
        collector.put(data, response, 'b', 'b2')

        assert data.get('a') == 1
        assert data.get('b2') == 2

        response['a'] = 3

        collector.fill(data, response, 'a')

        assert data.get('a') == 1


class TestRdapDataCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args, data):
        requester = RdapRequester(CountryDetector())

        collector = RdapCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'RDAP'
        assert success is True
        assert response_time > 0
        assert response['country'] == 'US'
        assert response['port43'] == 'whois.arin.net'
        assert collector.get_cidr(response) == '8.8.8.0/24'

        assert ('country' in collector.get_requires())

    def test_request_data(self, args):
        requester = RdapRequester(CountryDetector())

        collector = RdapCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response = collector.request_data(target, [])

        assert success is True
        assert response['country'] == 'US'
        assert response['port43'] == 'whois.arin.net'

    def test_build_data(self, args, data):
        requester = RdapRequester(CountryDetector())
        collector = RdapCollector(requester, args)

        target = Target()

        collector.build_data(target, data, False, {'cidr': 'abc'}, 8)
        assert data.get('cidr') == 'abc'


class TestDnsTxtCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = DnsResolveRequester()

        collector = DnsTxtCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'DNSTXT'
        assert success is True
        assert response_time >= 0
        assert response['country'] == 'US'
        assert collector.get_cidr(response) == '8.8.8.0/24'

        assert ('country' in collector.get_requires())

    def test_request_data(self, args):
        requester = DnsResolveRequester()

        collector = DnsTxtCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response = collector.request_data(target, [])

        assert success is True
        assert response['country'] == 'US'

    def test_build_data(self, args, data):
        requester = DnsResolveRequester()
        collector = DnsTxtCollector(requester, args)

        target = Target()

        collector.build_data(target, data, False, {'cidr': 'abc'}, 8)
        assert data.get('cidr') == 'abc'


class TestIpInfoCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = IpInfoRequester()

        collector = IpInfoCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert success is True
        assert response_time >= 0
        assert response['country'] == 'US'
        assert response['timezone'] == 'America/Los_Angeles'
        assert collector.get_cidr(response) is None

        assert ('country' in collector.get_requires())

    def test_request_data(self, args):
        requester = IpInfoRequester()

        collector = IpInfoCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response = collector.request_data(target, [])

        assert collector.get_name() == 'IPINFO'
        assert success is True
        assert response['country'] == 'US'
        assert response['timezone'] == 'America/Los_Angeles'

    def test_build_data(self, args, data):
        requester = IpInfoRequester()
        collector = IpInfoCollector(requester, args)

        target = Target()

        collector.build_data(target, data, False, {'country': 'abc'}, 8)
        assert data.get('country') == 'abc'


class TestDnsReverseCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = DnsResolveRequester()

        collector = DnsReverseCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'DNSREVERSE'
        assert success is True
        assert response_time >= 0
        assert response['hostname'] == 'dns.google'

        assert ('hostname' in collector.get_requires())

    def test_request_data(self, args):
        requester = DnsResolveRequester()

        collector = DnsReverseCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response = collector.request_data(target, [])

        assert success is True
        assert response['hostname'] == 'dns.google'

    def test_build_data(self, args, data):
        requester = DnsResolveRequester()
        collector = DnsReverseCollector(requester, args)

        target = Target()

        collector.build_data(target, data, False, {'hostname': 'abc'}, 8)
        assert data.get('hostname') == 'abc'


class TestSelfCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = IpInfoRequester()
        dns_resolver = DnsResolveRequester()
        server_reactivity = ServerReactivity()

        collector = SelfCollector(requester, dns_resolver, server_reactivity, args)

        target = Target()

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'SELF_INFO'
        assert collector.get_requires() == []

        assert success is True
        assert response_time >= 0
        assert len(response['country']) == 2
        assert len(response['region']) >= 0

    def test_request_data(self, args):
        requester = IpInfoRequester()
        dns_resolver = DnsResolveRequester()
        server_reactivity = ServerReactivity()

        collector = SelfCollector(requester, dns_resolver, server_reactivity, args)

        target = Target()

        success, response = collector.request_data(target, [])

        assert success is True

    def test_build_data(self, args):
        requester = IpInfoRequester()
        dns_resolver = DnsResolveRequester()
        server_reactivity = ServerReactivity()

        collector = SelfCollector(requester, dns_resolver, server_reactivity, args)

        target = Target()
        data = ValueData({})

        collector.build_data(target, data, True, {}, 8.0)

        assert data.get('ip') is None


class TestPassDataCollector:
    @pytest.fixture(autouse=True)
    def data(self):
        data = ValueData({})

        return data

    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request_data(self, args):
        requester = PassRequester()

        success, response = requester.request(None)

        assert success is False
        assert response == {}

    def test_request(self, args, data):
        requester = PassRequester()

        collector = PassDataCollector('PASS', requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response = requester.request(target)

        assert success is False
        assert requester.get_host() is None

        success, response, response_time = collector.request(target, [])

        assert success is False
        assert collector.initialize({}) is None
        assert collector.get_name() == 'PASS'
        assert collector.get_requires() == []
        assert collector.build_data(target, data, False, {}, 8) is None
        assert success is False

        success, response = collector.request_data(target, [])

        assert success is False

        success, response = collector.request_data(target, [])

        assert success is False

    def test_request2(self, mocker, args):
        requester = mocker.Mock(spec=PassRequester)
        requester.request.return_value = True, {'a': 1}

        collector = PassDataCollector('PASS', requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'PASS'
        assert success is True
        assert response['a'] == 1


class TestICMPCollector:
    @pytest.fixture(autouse=True)
    def setup(self, monkeypatch):
        monkeypatch.setattr('subprocess.check_output', lambda v, universal_newlines=True: 0)

    def test_request(self, args, monkeypatch):
        requester = ServerReactivity()

        monkeypatch.setattr(ServerReactivity, 'request_icmp', lambda *args, **kwargs: True)

        collector = ICMPCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'ICMP'
        assert success is True

        assert collector.get_requires()[0] == 'icmp'

    def test_build_data(self, args):
        requester = ServerReactivity()

        collector = ICMPCollector(requester, args)

        target = Target()
        data = ValueData({})
        collector.build_data(target, data, True, {}, 8.0)

        assert data.get('icmp') is True


class TestTCPCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = ServerReactivity()

        collector = TCPCollector(requester, args)

        target = Target()
        target.identifier = '1.1.1.1'
        target.ip = '1.1.1.1'
        target.port = 80

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'TCP'
        assert success is True

        assert collector.get_requires()[0] == 'tcp'

    def test_build_data(self, args):
        requester = ServerReactivity()

        collector = TCPCollector(requester, args)

        target = Target()
        data = ValueData({})
        collector.build_data(target, data, True, {}, 8.0)

        assert data.get('tcp') is True


class TestUDPCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args):
        requester = ServerReactivity()

        collector = UDPCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'
        target.port = 53

        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'UDP'
        assert success is True

        assert collector.get_requires()[0] == 'udp'

    def test_build_data(self, args):
        requester = ServerReactivity()

        collector = UDPCollector(requester, args)

        target = Target()
        data = ValueData({})
        collector.build_data(target, data, True, {}, 8.0)

        assert data.get('udp') is True


class TestHttpCollector:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_request(self, args, caplog):
        requester = HttpRequester()

        collector = HttpCollector(requester, args)

        target = Target()
        target.url = 'https://www.wikipedia.org/'

        collector.http = 2
        success, response, response_time = collector.request(target, [])

        assert collector.get_name() == 'HTTP'
        assert success is True
        assert response['http_status'] == 200
        assert ('http_status' in collector.get_requires())

        with caplog.at_level(logging.INFO):
            collector.request(target, [])

        assert re.search(r'HTTP_TIME', caplog.text)

    def test_build_data(self, args):
        requester = HttpRequester()

        collector = HttpCollector(requester, args)

        target = Target()
        data = ValueData({})
        collector.build_data(target, data, True, {}, 8.0)

        assert data.get('http') is True



import pytest
import re

from ipsurv.serializer.serializer import Serializer
from ipsurv.serializer.line_serializer import LineSerializer
from ipsurv.serializer.json_serializer import JsonSerializer
from ipsurv.core.entity import ValueData, Target


@pytest.fixture
def args(mocker):
    args = mocker.Mock()
    args.fixed_format = '{ip}'
    args.fixed_format_params = ['ip']
    args.fixed_delimiter = ','
    args.alt_delimiter = ';'
    args.fixed_enclose = ''
    args.exhaustive = False

    return args


class TempSerializerClass(Serializer):
    def build_row(self, data):
        # type: (ValueData) -> object
        return None

    def build_error(self, error):
        return None


class TestSerializer:
    @pytest.fixture(autouse=True)
    def setup(self, args):
        self.serializer = TempSerializerClass(args)

    def test_set_survey_mode(self):
        self.serializer.set_survey_mode(1)

        assert self.serializer.survey_mode == 1

    def test_set_delimiter(self):
        self.serializer.set_delimiter(',')
        assert self.serializer.delimiter == ','

    def test_output_begin(self, args):
        assert self.serializer.output_begin(1, args, []) is None

    def test_create_labels(self):
        values = self.serializer.create_labels({'a': 1}, 1)

        assert values['a'] is not None

    def test_get_label(self):
        assert self.serializer.get_label('a', 1) == 'a'
        assert self.serializer.get_label('abc', 2) == 'Abc'

    def test_set_status(self, args):
        data = ValueData({'requests': []})
        target = Target('')

        target.identifier = 'abc'
        self.serializer.set_status(data, target, args, False)
        assert data.get('group_status') == '-'

        target.status = 'EXIST'
        self.serializer.set_status(data, target, args, True)
        assert data.get('status') == 'SKIP'

        self.serializer.set_status(data, target, args, False)
        assert data.get('status') == '-'

        args.group = False

        self.serializer.set_status(data, target, args, False)
        assert data.get('group_status') == '-'

    def test_transform(self):
        data = ValueData({'ip_type': None})
        assert self.serializer.transform(data) is None

    def test_filter_value(self):
        assert self.serializer.filter_value(123) is not None

    def test_output(self, capfd):
        self.serializer.output('test')

        captured = capfd.readouterr()

        assert re.search(r'test', captured.out)

    def test_output_result(self, capfd):
        self.serializer.output_result('test')

        captured = capfd.readouterr()

        assert re.search(r'test', captured.out)

    def test_output_complete(self, args):
        assert self.serializer.output_complete(1, args, []) is None

    def test_misc(self):
        assert self.serializer.transform_key_labels(None, None) is None
        assert self.serializer.output_message(None) is None
        assert self.serializer.output_item(None) is None


class TestLineSerializer:
    @pytest.fixture(autouse=True)
    def setup(self, args):
        self.serializer = LineSerializer(args)

    def test_transform(self):
        assert hasattr(self.serializer, 'transform')

    def test_transform_status(self):
        data = ValueData({})
        data.header = False

        data.set('icmp', True)
        data.set('tcp', True)
        data.set('udp', True)
        data.set('http', True)
        data.set('http_h2', 1)

        self.serializer.transform_status(data)

        assert data.get('icmp') == 'ICMP_OK'
        assert data.get('tcp') == 'TCP_OK'
        assert data.get('udp') == 'UDP_OK'
        assert data.get('http') == 'HTTP_OK'
        assert data.get('http_h2') == 'HTTP2'

    def test_filter_value(self):
        assert self.serializer.filter_value('test') == 'test'
        assert self.serializer.filter_value(True) == 'OK'
        assert self.serializer.filter_value('test,test') == 'test;test'
        assert self.serializer.filter_value(None) == ''

        target = Target('')
        target.identifier = 'abc'
        assert self.serializer.filter_value(target) == target

    def test_get_bool_status(self):
        assert self.serializer._get_bool_status(True) == 'OK'
        assert self.serializer._get_bool_status(False) == 'NG'

    def test_build_row(self):
        data = ValueData({'ip': 'IP'})

        assert self.serializer.build_row(data) == 'IP'

        with pytest.raises(Exception):
            self.serializer.format = '{sampe..'
            self.serializer.build_row(data)

    def test_build_error(self):
        v = self.serializer.build_error('test')

        assert v == 'ERROR,test'

    def test_output(self, capfd):
        self.serializer.output('test')

        captured = capfd.readouterr()

        assert re.search(r'^test\s*$', captured.out)

    def test_transform_key_labels(self):
        data = ValueData({'ip': 'ip'})

        self.serializer.transform_key_labels(data, 3)

        assert data.get('IP') is not None

    def test_output_message(self, capfd):
        self.serializer.output_message('test')

        captured = capfd.readouterr()

        assert re.search(r'^test\s*$', captured.out)

    def test_output_item(self, capfd):
        data = ValueData({'a': 1, 'b': 2, 'c': 3})

        self.serializer.output_item(data)

        captured = capfd.readouterr()

        assert re.search(r'a,1', captured.out)


class TestJsonSerializer:
    @pytest.fixture(autouse=True)
    def setup(self, args):
        self.serializer = JsonSerializer(args)

    def test_output_begin(self, args, capfd):
        self.serializer.output_begin(1, args, [])

        captured = capfd.readouterr()
        self.serializer.json_list = True

        assert re.search(r'\[', captured.out)

    def test_transform(self):
        data = ValueData({'ip_type': None})

        assert self.serializer.transform(data) is None

    def test_filter_value(self):
        v = self.serializer.filter_value(5)

        assert v == 5

    def test_build_row(self):
        values = self.serializer.build_row(ValueData({'ip': 'IP', 'test': 1}))

        assert values == {'ip': 'IP'}

        self.serializer.exhaustive = False
        values = self.serializer.build_row(ValueData({'ip': 'IP', 'test': 1}))

        assert values == {'ip': 'IP'}

    def test_build_error(self):
        values = self.serializer.build_error('aaa')

        assert values['error'] == 'aaa'

    def test_output_complete(self, args, capfd):
        self.serializer.output_complete(1, args, [])

        captured = capfd.readouterr()
        self.serializer.json_list = True

        assert re.search(r'\]', captured.out)

    def test_output(self, capfd):
        self.serializer.output({'a': 1})

        captured = capfd.readouterr()

        assert re.search(r'"a":\s*1', captured.out)

        self.serializer.json_list = True
        self.serializer.output({'a': 1})

        captured = capfd.readouterr()

        assert re.search(r'"a":\s*1', captured.out)

    def test_output_message(self):
        assert self.serializer.output_message(None) is None

    def test_transform_key_labels(self):
        assert self.serializer.transform_key_labels(None, 1) is None

    def test_output_item(self, capfd):
        data = ValueData({'ip': 'IP'})

        self.serializer.output_item(data)
        captured = capfd.readouterr()

        assert re.search(r'ip', captured.out)



import logging
import re
import urllib.parse
from abc import ABC

from ipsurv.configs import Constant
from ipsurv.core.entity import Target
from ipsurv.core.entity import ValueData
from ipsurv.core.pipeline import Pipeline
from ipsurv.requester.dns_resolver import DnsResolveRequester
from ipsurv.util.sys_util import System
import ipaddress


class TargetParser(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#targetparser
    """
    def __init__(self, args, pipeline, dns_resolver):
        """
        :param args:
        :type args: argparse.Namespace
        :param pipeline:
        :type pipeline: Pipeline
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        """
        self.pipeline = pipeline  # type: Pipeline
        self.autodetect = args.autodetect  # type: bool
        self.identify_int = args.identify_int  # type: bool
        self.ranges = args.fixed_ranges  # type: list
        self.dns_resolver = dns_resolver  # type: DnsResolveRequester

    def parse(self, data, original, args):
        # type: (ValueData, str, object) -> Target
        """
        :param data:
        :type data: ValueData
        :param original:
        :type original: str
        :param args:
        :type args: argparse.Namespace
        :rtype: Target
        """
        target = self._parse_target(data, original, args)

        identify = self.pipeline.pre_target_identify(data, target)

        if identify:
            self._identify_target(data, target, args)

        if target.status != Constant.STATUS_EXIST:
            target.identifier = target.status

        self._assign_data_target(data, target)

        if System.is_logging():
            logging.log(logging.DEBUG, 'TARGET_RAW:' + target.raw)
            logging.log(logging.INFO, 'TARGET_IDENTIFIER:' + str(target.identifier))

            System.output_data('TARGET_DATA', target.get_values())

        return target

    def _parse_target(self, data, original, args):
        # type: (ValueData, str, object) -> Target
        """
        :param data:
        :type data: ValueData
        :param original:
        :type original: str
        :param args:
        :type args: argparse.Namespace
        :rtype: Target
        """

        if self.autodetect:
            raw = self._detect_target_raw(original, args)
        else:
            raw = original

        return self._create_target(raw.strip())

    def _identify_target(self, data, target, args):
        # type: (ValueData, Target, object) -> None
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        :param args:
        :type args: argparse.Namespace
        """

        if target.raw:
            target.identified = self._identify_target_ip(data, target, args)
        else:
            target.status = Constant.STATUS_EMPTY

        if target.identified:
            target.status = Constant.STATUS_EXIST

            self._prepare_target_data(data, target)

        logging.info('IP:' + str(target.ip))
        logging.info('FQDN:' + str(target.fqdn))
        logging.info('PORT:' + str(target.port))

    def _prepare_target_data(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        """
        ip_address = ipaddress.ip_address(target.identifier)
        target.identifier_int = int(ip_address)

        data.set('ip', target.ip)
        data.set('ip_int', target.identifier_int)
        data.set('ip_hex', '.'.join(f'{v:02x}' for v in ip_address.packed).upper())
        data.set('ip_reversed', '.'.join(reversed(ip_address.exploded.split('.'))))
        data.set('port', target.port)

        self._evaluate(data, target)

    def _identify_ip_int(self, raw):
        def convert_ip(match):
            ip_int = int(match.group(1))
            return str(ipaddress.ip_address(ip_int))

        return re.sub(r'(?<!\d)(\d{8,10})(?!\d)', convert_ip, raw)

    def _identify_target_ip(self, data, target, args):
        # type: (ValueData, Target, object) -> bool

        identified = False

        netloc = self._find_ip(target.raw, True)

        if not netloc:
            url = self._find_url(target.raw)

            if url:
                target.url = url

                parsed = urllib.parse.urlparse(url)
                netloc = parsed.netloc
            else:
                if not self.identify_int:
                    netloc = target.raw
                else:
                    netloc = self._identify_ip_int(target.raw)

        (fqdn_ip, port) = self._split_port(netloc)

        target.port = port

        if fqdn_ip:
            ip = self._find_ip(fqdn_ip, False)

            if not ip:
                fqdn = self._find_fqdn(fqdn_ip)

                target.fqdn = fqdn

                if fqdn:
                    if args.resolve:
                        try:
                            ip = self.dns_resolver.resolve_ip(fqdn)
                        except Exception:
                            target.status = Constant.STATUS_RESOLVE_FAIL
                else:
                    target.status = Constant.STATUS_ILLEGAL_FORMAT

            target.identifier = target.ip = ip

            if ip:
                identified = True

        return identified

    def _evaluate(self, data, target):
        # type: (ValueData, Target) -> None

        ip_address = ipaddress.ip_address(target.ip)

        self._evaluate_ip_type(data, ip_address)

        if len(self.ranges) > 0:
            self._evaluate_in_ranges(data, ip_address)

    def _evaluate_ip_type(self, data, ip_address):
        ip_type = Constant.IP_TYPE_PRIVATE if ip_address.is_private else Constant.IP_TYPE_PUBLIC

        data.set('ip_type', ip_type)

    def _evaluate_in_ranges(self, data, ip_address):
        # type: (ValueData, Target) -> None

        in_range = False

        for range in self.ranges:
            network = ipaddress.ip_network(range, strict=False)

            if ip_address in network:
                in_range = True
                break

        data.set('in_range', in_range)

    def _assign_data_target(self, data, target):
        # type: (ValueData, Target) -> None
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: Target
        """

        data.set('identifier', target.identifier)
        data.set('identifier_int', target.identifier_int)

        data.set('target', target)

    def _detect_target_raw(self, original, args):
        rows = re.split(args.fixed_delimiter, original)

        for row in rows:
            if self._find_url(row) or self._find_fqdn(row) or self._find_ip(row):
                return row

        return ''

    def _split_port(self, v):
        vals = v.split(':')
        port = None

        if len(vals) == 2:
            fqdn_ip = vals[0]

            try:
                port = int(vals[1])
            except Exception:
                pass
        else:
            fqdn_ip = vals[0]

        return fqdn_ip, port

    def _find_url(self, v):
        m = re.search(r'(https?:\/\/|\/\/)[a-z0-9][a-z0-9.\-/?=&_%!+:]+', v, flags=re.IGNORECASE)

        return m.group() if m is not None else None

    def _find_fqdn(self, v):
        m = re.search(r'([a-z0-9\-]{1,128}(?<!\-)\.)+[a-z0-9]{2,}', v, flags=re.IGNORECASE)

        return m.group() if m is not None else None

    def _find_ip(self, v, with_port=False):
        regex = r'[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}'

        if with_port:
            regex += r'(:\d{2,5})?'

        m = re.search(regex, v)

        return m.group() if m is not None else None

    def _create_target(self, raw):
        return Target(raw.strip())



from abc import ABC

from ipscap.configs import Config, Constant


class Pipeline(ABC):
    """

    """

    def __init__(self):
        self.config = None  # type: Config

    def initialize(self, config):
        """
        :param config:
        :type config: Config
        """

        self.config = config

    def init_configure(self, arguments):
        """
        :param arguments:
        :type arguments: dict
        """
        pass

    def pass_eth_ips(self, eth_ips):
        pass

    def pre_recieve_loop(self, eth_socket, ip_header_parser):
        pass

    def pass_eth_header(self, raw_data, eth_header):
        pass

    def pass_ip_header(self, mtu_data, ip_header):
        pass

    def pass_header_parser(self, ip_header, header_parser):
        return header_parser

    def pass_protocol_header(self, ip_header, protocol_header):
        pass

    def verify_capture(sself, packet_filter, ip_header, protocol_header, is_capture):
        return is_capture

    def process_captured_transfer(self, ip_header, protocol_header, passage_num):
        pass

    def complete(self, transfers):
        pass

    def get_filename(self, ip_header, protocol_header, filename):
        return filename

    def pre_writefile(self, ip_header, protocol_header, file):
        pass

    def post_writefile(self, ip_header, protocol_header, file):
        pass



import pytest
from unittest.mock import patch
from ipsurv.__main__ import main
import re
import sys
import io
import os


class TestMain:
    def test_main(self, capsys, monkeypatch):
        monkeypatch.setattr(sys, 'argv', ['ipsurv.py'])

        with patch('ipsurv.util.args_util.StdinLoader.read_stdin', return_value=[]):
            main()

            captured = capsys.readouterr()

            assert re.search(r'No target data', captured.out)



import pytest

from ipsurv.core.object_factory import ObjectFactory


class TestObjectFactory:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.object_factory = ObjectFactory()

    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.fixed_format_params = []
        args.fixed_timeout = {'dns': 0, 'http': 0, 'reactivity': 0}
        args.fixed_collectors = []
        args.autodetect = False

        return args

    def test_get_config(self):
        obj = self.object_factory.get_config()

        assert obj.__name__ == 'Config'

    def test_create_pipeline(self):
        obj = self.object_factory.create_pipeline()

        assert obj.__class__.__name__ == 'Pipeline'

    def test_create_value_data_factory(self, args):
        config = self.object_factory.get_config()

        obj = self.object_factory.create_value_data_factory(args, config)

        assert obj.__class__.__name__ == 'ValueDataFactory'

    def test_create_args_builder(self):
        obj = self.object_factory.create_args_builder(None, None)

        assert obj.__class__.__name__ == 'ArgsBuilder'

    def test_create_target_parser(self, args):
        obj = self.object_factory.create_target_parser(args, None, None)

        assert obj.__class__.__name__ == 'TargetParser'

    def test_create_dns_resolver(self, args):
        obj = self.object_factory.create_dns_resolver(args)

        assert obj.__class__.__name__ == 'DnsResolveRequester'

    def test_create_collectors(self, args):
        obj = self.object_factory.create_collectors(args, None)

        assert isinstance(obj, dict)

    def test_create_rdap_collector(self, args):
        obj = self.object_factory.create_rdap_collector(args)

        assert obj.__class__.__name__ == 'RdapCollector'

    def test_create_dnstxt_collector(self, args):
        obj = self.object_factory.create_dnstxt_collector(None, args)

        assert obj.__class__.__name__ == 'DnsTxtCollector'

    def test_create_ipinfo_collector(self, args):
        obj = self.object_factory.create_ipinfo_collector(args)

        assert obj.__class__.__name__ == 'IpInfoCollector'

    def test_create_self_collector(self, args):
        obj = self.object_factory.create_self_collector(args, None, None)

        assert obj.__class__.__name__ == 'SelfCollector'

    def test_create_dns_reverse_collector(self, args):
        obj = self.object_factory.create_dns_reverse_collector(None, args)

        assert obj.__class__.__name__ == 'DnsReverseCollector'

    def test_create_reactivities(self, args):
        obj = self.object_factory.create_reactivities(args)

        assert isinstance(obj, list)

    def test_create_server_reactivity(self, args):
        obj = self.object_factory.create_server_reactivity(args)

        assert obj.__class__.__name__ == 'ServerReactivity'

    def test_create_http(self, args):
        obj = self.object_factory.create_http(args)

        assert obj.__class__.__name__ == 'HttpRequester'

    def test_create_icmp_collector(self, args):
        obj = self.object_factory.create_icmp_collector(None, args)

        assert obj.__class__.__name__ == 'ICMPCollector'

    def test_create_tcp_collector(self, args):
        obj = self.object_factory.create_tcp_collector(None, args)

        assert obj.__class__.__name__ == 'TCPCollector'

    def test_create_udp_collector(self, args):
        obj = self.object_factory.create_udp_collector(None, args)

        assert obj.__class__.__name__ == 'UDPCollector'

    def test_create_http_collector(self, args):
        obj = self.object_factory.create_http_collector(None, args)

        assert obj.__class__.__name__ == 'HttpCollector'

    def test_create_serializer_line(self, args):
        args.json = False

        obj = self.object_factory.create_serializer(args)

        assert obj.__class__.__name__ == 'LineSerializer'

    def test_create_serializer_json(self, args):
        args.json = True

        obj = self.object_factory.create_serializer(args)

        assert obj.__class__.__name__ == 'JsonSerializer'



import json

from ipsurv.serializer.serializer import Serializer


class JsonSerializer(Serializer):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#serializer
    """

    def __init__(self, args):
        super().__init__(args)

        self.format_params = args.fixed_format_params
        self.json = args.json
        self.json_list = args.json_list
        self.exhaustive = args.exhaustive

    def output_begin(self, mode, args, rows):
        if self.json_list:
            print('[')

    def filter_value(self, v):
        return v

    def build_row(self, data):
        all_values = data.get_values()

        values = {}

        if not self.exhaustive:
            for param in self.format_params:
                values[param] = all_values[param]
        else:
            values = all_values

        return values

    def build_error(self, error):
        return {'error': error}

    def output_complete(self, mode, args, rows):
        if self.json_list:
            print(']', flush=True)

    def output(self, v):
        if self.json == 2:
            r = json.dumps(v, indent=2)
        else:
            r = json.dumps(v)

        append = ',' if self.json_list else ''

        print(r + append, flush=True)

    def transform_key_labels(self, data, mode):
        pass

    def output_message(self, msg):
        pass

    def output_item(self, data):
        self.output(data.get_values())



import re
from abc import ABC


class Header(ABC):
    def __init__(self):
        self.header_data = b''
        self.checksum = 0

        self.payload_length = 0
        self.payload_data = b''

        self.raws = {}

    def get_sanitized_data(self):
        return re.sub(rb'^\x00+$', b'', self.payload_data)


class IPHeader(Header):
    PROTOCOL_ICMP = 1
    PROTOCOL_TCP = 6
    PROTOCOL_UDP = 17

    DIRECTION_SEND = 1
    DIRECTION_RECEIVE = 2

    @classmethod
    def get_protocol_code(cls, protocol):
        if protocol == IPHeader.PROTOCOL_ICMP:
            code = 'ICMP'
        elif protocol == IPHeader.PROTOCOL_TCP:
            code = 'TCP'
        elif protocol == IPHeader.PROTOCOL_UDP:
            code = 'UDP'
        else:
            code = 'UNKNOWN'

        return code

    @classmethod
    def get_protocol(cls, code):
        protocol = 0

        if code == 'ICMP':
            protocol = IPHeader.PROTOCOL_ICMP
        elif code == 'TCP':
            protocol = IPHeader.PROTOCOL_TCP
        elif code == 'UDP':
            protocol = IPHeader.PROTOCOL_UDP

        return protocol

    @classmethod
    def get_direction_code(cls, direction):
        if direction == IPHeader.DIRECTION_SEND:
            code = 'SEND'
        elif direction == IPHeader.DIRECTION_RECEIVE:
            code = 'RECEIVE'
        else:
            code = 'UNKNOWN'

        return code

    def __init__(self):
        super().__init__()

        self.version = 4
        self.iph_length = 0
        self.tos = 0
        self.total_length = 0
        self.identification = 0
        self.flags = 0
        self.fragment_offset = 0
        self.ttl = 0
        self.protocol = 0
        self.protocol_code = None
        self.src_ip = None
        self.src_ip_int = None
        self.dest_ip = None
        self.dest_ip_int = None

        self.direction = 0
        self.direction_code = None


class ProtocolHeader(ABC):
    def __init__(self):
        self.src_port = 0
        self.dest_port = 0


class ICMPHeader(Header, ProtocolHeader):
    DEFAULT_HEADER_LEN = 4

    def __init__(self):
        super().__init__()

        self.src_port = -1
        self.dest_port = -1

        self.icmp_type = 0
        self.code = 0


class TCPHeader(Header, ProtocolHeader):
    DEFAULT_HEADER_LEN = 20

    FLAG_FIN = 1
    FLAG_SYN = 2
    FLAG_RST = 4
    FLAG_PSH = 8
    FLAG_ACK = 16
    FLAG_URG = 32
    FLAG_ECE = 64
    FLAG_CWR = 128

    @classmethod
    def get_flag_codes(cls, flags):
        flag_codes = []

        if flags & TCPHeader.FLAG_FIN:
            flag_codes.append('FIN')

        if flags & TCPHeader.FLAG_SYN:
            flag_codes.append('SYN')

        if flags & TCPHeader.FLAG_RST:
            flag_codes.append('RST')

        if flags & TCPHeader.FLAG_PSH:
            flag_codes.append('PSH')

        if flags & TCPHeader.FLAG_ACK:
            flag_codes.append('ACK')

        if flags & TCPHeader.FLAG_URG:
            flag_codes.append('URG')

        if flags & TCPHeader.FLAG_ECE:
            flag_codes.append('ECE')

        if flags & TCPHeader.FLAG_CWR:
            flag_codes.append('CWR')

        return flag_codes

    @classmethod
    def get_flags(cls, codes):
        flags = 0

        if 'FIN' in codes:
            flags |= TCPHeader.FLAG_FIN

        if 'SYN' in codes:
            flags |= TCPHeader.FLAG_SYN

        if 'RST' in codes:
            flags |= TCPHeader.FLAG_RST

        if 'PSH' in codes:
            flags |= TCPHeader.FLAG_PSH

        if 'ACK' in codes:
            flags |= TCPHeader.FLAG_ACK

        if 'URG' in codes:
            flags |= TCPHeader.FLAG_URG

        if 'ECE' in codes:
            flags |= TCPHeader.FLAG_ECE

        if 'CWR' in codes:
            flags |= TCPHeader.FLAG_CWR

        return flags

    def __init__(self):
        super().__init__()

        self.seq_no = 0
        self.ack_no = 0
        self.tcph_length = 0
        self.flags = 0
        self.flag_codes = []
        self.window = 0
        self.urgent_pointer = 0
        self.tcp_options = 0


class UDPHeader(Header, ProtocolHeader):
    DEFAULT_HEADER_LEN = 8

    def __init__(self):
        super().__init__()

        self.udph_length = 0



from collections import OrderedDict

from ipsurv.core.pipeline import Pipeline
from ipsurv.serializer.serializer import Serializer


class SurveySelf:
    def __init__(self, args, config, data_factory, collector, pipeline, serializer):
        self.config = config
        self.data_factory = data_factory
        self.collector = collector

        self.pipeline = pipeline  # type: Pipeline
        self.serializer = serializer  # type: Serializer

        serializer.set_delimiter(': ')

    def initialize(self):
        self.serializer.output_message(self.config.HEAD_MSG_SELF + '\n')

    def dispatch(self):
        data = self.data_factory.create()

        success = self._request(data)

        self._modify_orders(data)

        self._output_data(success, data)

    def _request(self, data):
        success, response, response_time = self.collector.request(None, [])

        self.collector.build_data(None, data, success, response, response_time)

        return success

    def _modify_orders(self, data):
        orders = ['ip', 'hostname', 'organization', 'country', 'city_name', 'region_name', 'postal', 'geo', 'timezone', 'local_ip', 'local_dns']

        values = data.get_data()

        data.set_data(OrderedDict((k, values[k]) for k in orders))

    def _output_data(self, success, data):
        if success:
            self.pipeline.output_result_self(data)
        else:
            msg = self.pipeline.build_error('Data not found or Error occurred.')

            self.pipeline.output_result(msg)



import pytest
from ipsurv.core.pipeline import Pipeline
from ipsurv.core.entity import Target, ValueData
from ipsurv.serializer.line_serializer import LineSerializer
from ipsurv.configs import Config
import re


class TestPipeline:
    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.fixed_format = '{ip}'
        args.fixed_format_params = ['ip']
        args.fixed_delimiter = ','
        args.alt_delimiter = ';'
        args.fixed_enclose = ''
        args.exhaustive = False

        return args

    @pytest.fixture(autouse=True)
    def target(self, args):
        target = Target()
        target.identifier_int = 123

        return target

    @pytest.fixture(autouse=True)
    def data(self):
        keys = Config.FORMAT_PARAMS
        data = ValueData(dict(zip(keys, keys)))

        return data

    @pytest.fixture(autouse=True)
    def test_setup(self, args):
        self.pipeline = Pipeline()

        self.pipeline.initialize(None, LineSerializer(args))

    def test_initialize(self):
        assert self.pipeline.initialize(None, None) is None

    def test_init_configure(self):
        assert self.pipeline.init_configure({}, {}) is None

    def test_pre_configure(self):
        assert self.pipeline.pre_configure({}, {}, {}) is None

    def test_post_configure(self):
        assert self.pipeline.post_configure({}, {}, {}) is None

    def test_detect_survey_mode(self):
        assert self.pipeline.detect_survey_mode(1) == 1

    def test_begin_process(self, args):
        assert self.pipeline.begin_process(1, args, []) is None

    def test_pre_target_parse(self):
        data = ValueData(None)
        assert self.pipeline.pre_target_parse(data, '123') == '123'

    def test_pre_target_identify(self):
        assert self.pipeline.pre_target_identify(None, None) is True

    def test_pre_output_headers(self):
        assert self.pipeline.pre_output_headers(None) is None

    def test_pre_collect(self, data, target, args):
        assert self.pipeline.pre_collect(data, target, args) is None

    def test_find_group(self):
        assert self.pipeline.find_group(None, None) is None

    def test_get_group_identify(self, target):
        assert self.pipeline.get_group_identify(None, target) == 123

    def test_create_group(self, data, target):
        assert self.pipeline.create_group(data, target, '8', '') is None

    def test_pre_request(self):
        assert self.pipeline.pre_request(None, None, None) is None

    def test_post_request(self):
        assert self.pipeline.post_request(None, None, None, None, None) is None

    def test_post_collect(self, data, target):
        assert self.pipeline.post_collect(data, target, None, False) is None

    def test_build(self, data, target):
        assert type(self.pipeline.build(data, target)) is str

    def test_build_error(self, capfd):
        v = self.pipeline.build_error('123')

        assert re.search(r'123', v)

    def test_output_result(self, capfd):
        self.pipeline.output_result('123')
        captured = capfd.readouterr()
        assert re.search(r'123', captured.out)

    def test_output_result_self(self):
        assert hasattr(self.pipeline, 'output_result_self')

    def test_complete_process(self, args, capfd):
        assert self.pipeline.complete_process(1, args, ['123']) is None



from distutils.util import strtobool


class Constant:
    APP_NAME = 'ipsurv'

    APP_DESCRIPTION = '"IpSurv" is a command-line program for surveying IP addresses, host information, and more. Also "IpSurv" is extensible program by Python.'
    APP_BOTTOM_DESC = '''command examples:
  ipsurv 192.168.1.100
  ipsurv 192.168.1.100 192.168.1.101
  ipsurv google.com wikipedia.org
  ipsurv google.com --format=web --http=1
  cat list.txt|ipsurv
  cat list.txt|ipsurv --format="hostname"
  cat list.txt|ipsurv --json=2
  cat /var/log/httpd/access_log|ipsurv --add_ip --no_original
  ipsurv self

documents:
  Documentation site: https://deer-hunt.github.io/ipsurv/

  PyPI: https://pypi.org/project/ipsurv/
  Github: https://github.com/deer-hunt/ipsurv/

bundled tool:
  `ipscap`: Packet capture tool which support "ICMP, TCP, UDP" protocol. e.g. `ipscap --help`
'''

    ENV_ARGS_VAR = 'IPSURV_ARGS'
    ENV_CONF_VAR = 'IPSURV_CONF'

    MODE_SURVEY_IPS = 1
    MODE_SURVEY_SELF = 2

    STR_LOWER = 1
    STR_PASCAL = 2
    STR_UPPER = 3

    DELIMITER_DEFAULT = ','

    STATUS_EXIST = 'EXIST'
    STATUS_EMPTY = 'EMPTY'
    STATUS_RESOLVE_FAIL = 'RESOLVE_FAIL'
    STATUS_ILLEGAL_FORMAT = 'ILLEGAL_FORMAT'

    IP_TYPE_PRIVATE = 1
    IP_TYPE_PUBLIC = 2


class Config:
    PRE_ARGUMENTS = {
        'verbose': {'default': 0, 'type': int, 'help': 'Verbose mode. [Level - 1:TRACE_ERROR, 2:INFO, 3:DEBUG]', 'choices': [0, 1, 2, 3]},
        'debug': {'default': False, 'help': '`--debug` is equivalent to `--verbose=3`.', 'action': 'store_true'},
        'log': {'default': None, 'type': str, 'help': 'Verbose log filename.', 'metavar': '{string}'},
        'disable_env': {'default': False, 'help': 'Disable to load env variable for args. Env name: `IPSURV_ARGS`.', 'action': 'store_true'}
    }

    ARGUMENTS = {
        'resolve': {'default': True, 'type': strtobool, 'help': 'Resolve the name to IP if target value is domain or hostname automatically.', 'choices': [0, 1]},
        'identify_int': {'default': False, 'help': 'Identify IP\'s int value.', 'action': 'store_true'},
        'autodetect': {'default': False, 'help': 'Autodetect an IP or hostname that is included in the line. [Experimental]', 'action': 'store_true'},
        'begin': {'default': -1, 'type': int, 'help': 'Beginning from sequence number.', 'metavar': '{number}'},
        'end': {'default': -1, 'type': int, 'help': 'Ending to sequence number.', 'metavar': '{number}'},

        'collect': {'default': 'rdap;dnstxt;dnsreverse;ipinfo;geoip', 'type': str, 'help': 'Data collectors. See reference manual in detail. ex: rdap;dnstxt;dnsreverse;ipinfo;geoip', 'metavar': '{string}'},
        'all_collect': {'default': False, 'help': 'All data collectors.', 'action': 'store_true'},
        'timeout': {'default': '8.0', 'type': str, 'help': 'Timeout seconds. Specify single value - ex: 1,3.2. Or specify values of each connection types. "dns,http,reactivity" - "3;5.1;6"', 'metavar': '{string}'},

        'group': {'default': None, 'type': None, 'help': 'Grouping rule. ex: network, 24, 255.255.255.0', 'metavar': '{string}'},
        'skip_duplicate': {'default': 0, 'type': int, 'help': 'Skip duplicate group. *2: It also skip checking server reactivity[icmp, tcp, udp].', 'choices': [0, 1, 2]},
        'range': {'default': '', 'type': str, 'help': 'Check whether IP is in IP/subnet ranges.  The value is CIDR notation. ex: "1.0.0.1/8;192.168.1.1/24"', 'metavar': '{string}'},

        'format': {'default': 'default', 'type': None, 'help': 'Output format. Specify `Profile` or `Parameter`. See reference manual in detail. ex: simple, default, detail, heavy, geo, hostname etc.', 'action': 'StrAction', 'metavar': '{string}'},
        'no_original': {'default': False, 'help': 'Cancel outputting the original line automatically.', 'action': 'store_true'},
        'sequence': {'default': False, 'help': 'Append sequence number.', 'action': 'store_true'},
        'add_ip': {'default': False, 'help': 'Append "ip" to the output format. For example, use when the target is a hostname, etc.', 'action': 'store_true'},
        'ident': {'default': False, 'help': 'Append identifier. Default identifier is ip.', 'action': 'store_true'},
        'enclose': {'default': None, 'type': str, 'help': 'Character of enclose in result line. If you specify "json" option, this option is disabled. ex: \'"\', "\'"', 'metavar': '{string}'},
        'delimiter': {'default': None, 'type': str, 'help': 'Delimiter-char in result line.', 'action': 'StrAction', 'metavar': '{string}'},
        'alt_delimiter': {'default': ';', 'type': str, 'help': 'Alternative delimiter character. If you specify "enclose" or "json" option, this option is disabled.', 'action': 'StrAction', 'metavar': '{string}'},
        'headers': {'default': 0, 'type': int, 'help': 'Show headers. 1: LowerCase, 2: PascalCase, 3: UpperCase', 'choices': [0, 1, 2, 3]},
        'json': {'default': 0, 'type': int, 'help': 'Output JSON data. *2: Output formatted JSON.', 'choices': [0, 1, 2]},
        'json_list': {'default': False, 'help': 'Output JSON list. It makes it easier to parse JSON.', 'action': 'store_true'},
        'exhaustive': {'default': False, 'help': 'Output exhaustive internal values in JSON. Use with "json" option.', 'action': 'store_true'},

        'icmp': {'default': False, 'type': strtobool, 'help': 'Check ICMP.', 'choices': [0, 1]},
        'tcp': {'default': 0, 'type': int, 'help': 'Check TCP port. Specify default port.', 'metavar': '{number}'},
        'udp': {'default': 0, 'type': int, 'help': 'Check UDP port. Specify default port.', 'metavar': '{number}'},
        'http': {'default': 0, 'type': int, 'help': 'Check HTTP response.', 'choices': [0, 1, 2]},

        'json_all': {'default': False, 'help': '`--json_all` is equivalent to `--json=2 --exhaustive`.', 'action': 'store_true'},
        'geoip_only': {'default': False, 'help': '`--geoip_only` is equivalent to `--collect=geoip --format=area`.', 'action': 'store_true'},
        'host_only': {'default': False, 'help': '`--host_only` is equivalent to `--collect=dnsreverse --format=hostname`.', 'action': 'store_true'},

        'version': {'default': False, 'help': 'Show version information.', 'action': 'store_true'}
    }

    ENV_CONFS = ['ipinfo_token', 'geoip']

    FORMAT_PROFILES = {
        'ip': ['ip'],
        'hostname': ['hostname'],
        'country': ['country'],
        'org': ['asn', 'org'],
        'address': ['country', 'address'],
        'timezone': ['timezone'],
        'network': ['cidr', 'network_start', 'network_end'],
        'geo': ['country', 'geo'],
        'area': ['continent', 'continent_name', 'country', 'country_name', 'timezone', 'geo'],
        'system': ['ip_type', 'ip_int', 'ip_hex', 'ip_reversed'],
        'web': ['http', 'http_status', 'http_size', 'http_server', 'http_mime', 'http_h2', 'http_time'],
        'simple': ['status', 'group', 'country'],
        'default': ['status', 'group', 'country', 'name', 'network_start', 'network_end'],
        'detail': ['status', 'group', 'country', 'name', 'handle', 'asn', 'org', 'cidr', 'geo', 'city_name', 'address', 'description', 'hostname', 'errors'],
        'heavy': ['status', 'group', 'country', 'timezone', 'name', 'handle', 'asn', 'org', 'cidr', 'network_start', 'network_end', 'ip_type', 'geo', 'city_name', 'region_name', 'address', 'description', 'hostname', 'errors']
    }

    FORMAT_PARAMS = [
        'success', 'status', 'requests', 'errors', 'identifier', 'identifier_int', 'target.*',
        'sequence', 'original', 'ip', 'ip_int', 'ip_hex', 'ip_reversed', 'port', 'ip_type', 'in_range',
        'group_int', 'group', 'group_found', 'group_status', 'network_start', 'network_end',
        'country', 'cidr',
        'rdap_time', 'port43', 'country_updated', 'name', 'handle', 'address', 'org', 'asn', 'timezone', 'description',
        'dnstxt_time', 'rir',
        'dnsreverse_time', 'hostname',
        'ipinfo_time', 'geo', 'postal', 'city_name', 'region_name',
        'continent', 'continent_name', 'subdivision', 'subdivision_name',
        'icmp', 'icmp_time', 'tcp', 'tcp_time', 'udp', 'udp_time',
        'http', 'http_time', 'http_status', 'http_size', 'http_server', 'http_mime', 'http_h2'
    ]

    MASTER_DATA = {
        'success': False, 'status': '', 'requests': [], 'errors': [],
        'sequence': None, 'original': None, 'target': None, 'ip': None, 'ip_int': None, 'ip_type': None, 'port': -1,
        'group_int': 0, 'group': '', 'group_found': False, 'group_status': '',
    }

    COLLECTORS = ['rdap', 'dnstxt', 'dnsreverse', 'ipinfo', 'geoip']

    HEAD_MSG_SELF = 'Self IP status by https://ipinfo.io'



import pytest

from ipsurv.util.sys_util import System
from ipsurv.util.network_util import DnsUtil
import socket
import re


class TestSystem:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_get_python_ver(self):
        ver = System.get_python_ver()

        assert ver > 2.0

    def test_verify_is(self):
        is_match = System.verify_os()

        assert not is_match

    def test_line(self, capfd):
        System.line('ABC')

        captured = capfd.readouterr()
        assert re.search("ABC", captured.out.strip())

    def test_warn(self, capfd):
        System.warn('ABC')

        captured = capfd.readouterr()
        assert re.search("ABC", captured.out.strip())


class TestDnsUtil:
    @pytest.fixture(autouse=True)
    def setup(self):
        pass

    def test_getaddrinfo(self):
        ip = DnsUtil.getaddrinfo('wikipedia.org')

        assert len(ip[0]) > 10

    def test_timeout(self):
        with pytest.raises((socket.timeout, socket.gaierror)):
            DnsUtil.getaddrinfo('ipsurv-2345253567456736533434563534.test', timeout=0.005)

    def test_resolve(self):
        ip = DnsUtil.resolve('wikipedia.org')

        assert len(ip) > 10

    def test_reverse(self):
        host = DnsUtil.reverse('8.8.8.8')

        assert host == 'dns.google'



from ipsurv.core.object_factory import ObjectFactory
from ipsurv.ip_surv_cmd import IpSurvCmd
from ipsurv.util.sys_util import System


def main():
    if System.get_python_ver() <= 3.2 and not System.load_module('ipaddress'):
        System.warn('"ipaddress" module is required. Please install by `pip install ipaddress`.')

    factory = ObjectFactory()

    ip_surv_cmd = IpSurvCmd(factory)

    ip_surv_cmd.run()


if __name__ == '__main__':
    main()



import os

from ipscap.util.raw_socket_entity import IPHeader
import logging


class DumpFile:
    def __init__(self, pipeline):
        self.dirname = None
        self.pipeline = pipeline

    def initialize(self, dirname):
        self.dirname = dirname

        if not os.path.exists(self.dirname):
            os.makedirs(self.dirname)

    def write(self, ip_header, protocol_header, append_header):
        filename = self.get_filename(ip_header, protocol_header)
        path = self.dirname + '/' + filename

        logging.log(logging.INFO, 'DUMPFILE_PATH: ' + path)

        with open(path, 'ab') as file:
            self.pipeline.pre_writefile(ip_header, protocol_header, file)

            if append_header:
                file.write(ip_header.header_data)
                file.write(protocol_header.header_data)

            file.write(protocol_header.payload_data)

            self.pipeline.post_writefile(ip_header, protocol_header, file)

        return path

    def get_filename(self, ip_header, protocol_header, ext='.dat'):
        protocol_code = IPHeader.get_protocol_code(ip_header.protocol).lower()

        if ip_header.direction == IPHeader.DIRECTION_SEND:
            port = protocol_header.src_port
            filename = protocol_code + '_' + ip_header.src_ip + '_' + str(protocol_header.src_port) + '_' + ip_header.dest_ip + '_' + str(protocol_header.dest_port)
        else:
            port = protocol_header.dest_port
            filename = protocol_code + '_' + ip_header.dest_ip + '_' + str(protocol_header.dest_port) + '_' + ip_header.src_ip + '_' + str(protocol_header.src_port)

        if port < 32768 and port != -1:
            filename = 'server_' + filename

        filename += '_' + ip_header.direction_code.lower() + ext

        filename = self.pipeline.get_filename(ip_header, protocol_header, filename)

        return filename

    def get_path(self):
        full_path = os.path.abspath(self.dirname)

        return full_path.rstrip('/') + '/'

    def get_file_num(self):
        full_path = self.get_path()

        files = next(os.walk(full_path))[2]

        return len(files)



from abc import ABC

from ipscap.configs import Config
from ipscap.core.pipeline import Pipeline
from ipscap.service.args_builder import ArgsBuilder
from ipscap.service.dumpfile import DumpFile
from ipscap.service.eth_socket import EthSocket
from ipscap.service.packet_filter import PacketFilter
from ipscap.service.protocol_service import ICMPProtocolService
from ipscap.service.protocol_service import TCPProtocolService
from ipscap.service.protocol_service import UDPProtocolService
from ipscap.service.transfer_store import TransferStore
from ipscap.service.view_helper import ViewHelper
from ipscap.util.raw_socket_parser import IPHeaderParser
from ipscap.util.evaluation_parser import EvaluationParser


class ObjectFactory(ABC):
    """

    """

    def get_config(self):
        """
        :rtype: Config
        """
        return Config

    def create_pipeline(self):
        """
        :rtype: Pipeline
        """
        return Pipeline()

    def create_args_builder(self, config, pipeline):
        """
        :param config:
        :type config: Config
        :param pipeline:
        :type pipeline: Pipeline
        :rtype: ArgsBuilder
        """
        ev_parser = self.create_evaluation_parser()

        return ArgsBuilder(config, pipeline, ev_parser)

    def create_evaluation_parser(self):
        return EvaluationParser()

    def create_eth_socket(self):
        return EthSocket()

    def create_ip_header_parser(self):
        return IPHeaderParser()

    def create_packet_filter(self, evaluation_parser):
        return PacketFilter(evaluation_parser)

    def create_transfer_store(self):
        return TransferStore()

    def create_dumpfile(self, pipeline):
        return DumpFile(pipeline)

    def create_view_helper(self):
        return ViewHelper()

    def create_icmp_protocol_service(self):
        return ICMPProtocolService()

    def create_tcp_protocol_service(self):
        return TCPProtocolService()

    def create_udp_protocol_service(self):
        return UDPProtocolService()



from ipscap.core.object_factory import ObjectFactory
from ipscap.ipscap_cmd import IpsCapCmd
from ipsurv.util.sys_util import System
import os


def main():
    if System.get_python_ver() <= 3.2 and not System.load_module('ipaddress'):
        System.exit('"ipaddress" module is required. Please install by `pip install ipaddress`.', True)

    if System.verify_os(windows=True, macos=True):
        System.exit('`ipscap` support only Unix/Linux OS.', True)

    factory = ObjectFactory()

    ips_cap_cmd = IpsCapCmd(factory)

    ips_cap_cmd.run()


if __name__ == '__main__':
    main()



from ipsurv.util.args_util import ArgValidator

from functools import partial

import re
import logging


class FormatValidator(ArgValidator):
    def __init__(self, profiles, params, debug=False):
        super().__init__(debug)

        self.name = 'format'
        self.profiles = profiles
        self.params = params

    def _validate(self, args):
        format = args.format

        if not format:
            raise self.arg_error('Empty value.')

        ft = format.lower()

        ft = ft if ft else 'default'

        if ft in self.profiles:
            format, params = self._create_by_profile(args, ft)
        else:
            format, params = self._parse_format(args, format)

            if len(params) == 0:
                raise self.arg_error('Format error.(' + args.format + ')')

        self._test_format(format, params)

        format, params = self._append_head(args, format, params)

        logging.log(logging.INFO, 'Fixed format:' + format)

        return format, params

    def _append_head(self, args, format, params):
        delimiter = args.fixed_delimiter

        if args.ident:
            format = '{identifier}' + delimiter + format
            params = ['identifier'] + params

        if args.add_ip:
            format = '{ip}' + delimiter + format
            params = ['ip'] + params

        if not args.no_original:
            format = '{original}' + delimiter + format
            params = ['original'] + params

        if args.sequence:
            format = '{sequence}' + delimiter + format
            params = ['sequence'] + params

        return format, params

    def _create_by_profile(self, args, profile):
        params = self.profiles[profile]

        if args.icmp:
            params.append('icmp')

        if args.tcp:
            params.append('tcp')

        if args.udp:
            params.append('udp')

        if args.http and profile != 'http':
            params.append('http')

        format = self._create_by_params(params, args)

        return format, params

    def _create_by_params(self, params, args):
        delimiter = args.fixed_delimiter

        params = self._filter_available_params(params, args)

        format = delimiter.join('{' + v + '}' for v in params)

        return format

    def _filter_available_params(self, params, args):
        def fn(v):
            if not args.group and v in ['group', 'group_int', 'group_found', 'group_status']:
                return False

            if not args.range and v == 'in_range':
                return False

            return True

        return list(filter(fn, params))

    def _parse_format(self, args, format):
        params = []

        format = re.sub(r'(\{([^{}]+?)\}|<([^<>]+?)>)', partial(self._parse_profile_param, defines=self.params, args=args, params=params), format)

        return format, params

    def _parse_profile_param(self, match, defines, args, params):
        org = match.group(0)
        param = match.group(2)
        profile = match.group(3)

        r = None

        if param:
            param = param.lower()

            if re.search(r'^[a-z0-9][a-z0-9_.]*$', param, re.IGNORECASE):
                params.append(param)
                r = org.lower()
            else:
                raise self.arg_error('Invalid param.(' + org + ')')
        else:
            profile = profile.lower()

            if profile in self.profiles:
                keys = self.profiles[profile]
                params.extend(keys)

                r = self._create_by_params(keys, args)
            else:
                raise self.arg_error('Unknown profile.(' + org + ')')

        return r

    def _test_format(self, format, params):
        data = {v: '' for v in params}

        class Dummy:
            def __getattr__(self, name):
                return None

        for k in params:
            m = re.search(r'^([^.]+)\.', k)

            if m:
                data[m.group(1)] = Dummy()

        try:
            format.format(**data)
        except Exception as e:
            raise self.arg_error('Format error.(' + str(e) + ')')


class TimeoutValidator(ArgValidator):
    def __init__(self, default=8.0, debug=False):
        super().__init__(debug)

        self.name = 'timeout'
        self.default = default

    def _validate(self, args):
        v = args.timeout

        tms = [self.default, self.default, self.default]

        if v is None:
            pass
        elif re.search(r'^[\d.]+$', v):
            t = self._parse_float(v)
            tms = [t for _ in tms]
        else:
            values = re.split(r'[,;:]', v)
            values = [self._parse_float(v) for v in values]

            for i in range(len(tms)):
                if i < len(values):
                    tms[i] = values[i]

        timeout = {'dns': tms[0], 'http': tms[1], 'reactivity': tms[2]}

        logging.log(logging.INFO, 'Fixed timeout:' + str(timeout))

        return timeout

    def _parse_float(self, v):
        v = v.strip()

        if not v:
            return self.default

        return float(v)



from ipscap.util.raw_socket_entity import TCPHeader


class Constant:
    PYPI_NAME = 'ipsurv'

    APP_NAME = 'ipscap'

    APP_DESCRIPTION = '`ipscap` captures "ICMP, TCP, UDP" packets. It supports filtering by various conditions, dumping file, displaying statistics.'
    APP_BOTTOM_DESC = '''command examples:
  ipscap --port="80;53" --find="GET"
  ipscap --port="80" --find="3\\d1"
  ipscap --find="HTTP/1.1 \\d01"
  ipscap --find="http" --find_mode=MATCH
  ipscap --find="00 99 f0 e0 78 4e 23 70 a1" --find_mode=HEX
  ipscap --find="Accept-Ranges: bytes\\r\\n\\r\\n\\x00\\x00\\x01\\x00\\x01\\x00" --find_mode=BINARY
  ipscap --find="HTTP" --tracking
  ipscap --condition="port!=22"
  ipscap --condition="port=80,443,53,-1" --protocol=TCP,UDP,ICMP
  ipscap --condition="src_port>=80;src_port<=500;flags=SYN,PSH"
  ipscap --condition="ttl>=120"
  ipscap --output=HEADER
  ipscap --output=BINARY --port="80,443"
  ipscap --output=LINE --port="80,443"
  ipscap --stat_mode=2 --protocol=TCP,UDP --output=NONE
  ipscap --port=80,443 --stat_group=1
  ipscap --port=80 --dumpfile=1
  ipscap --exclude_ssh
  ipscap --force


documents:
  Documentation site: https://deer-hunt.github.io/ipsurv/
  PyPI: https://pypi.org/project/ipsurv/
  Github: https://github.com/deer-hunt/ipsurv/
'''

    RECV_BUF_SIZE = 65565

    FIND_REGEX = 'REGEX'
    FIND_MATCH = 'MATCH'
    FIND_BINARY = 'BINARY'
    FIND_HEX = 'HEX'

    OUTPUT_NONE = 'NONE'
    OUTPUT_HEADER = 'HEADER'
    OUTPUT_TEXT = 'TEXT'
    OUTPUT_BINARY = 'BINARY'
    OUTPUT_BINARY_ALL = 'BINARY_ALL'
    OUTPUT_HEX = 'HEX'
    OUTPUT_HEX_ALL = 'HEX_ALL'
    OUTPUT_BASE64 = 'BASE64'
    OUTPUT_BASE64_ALL = 'BASE64_ALL'
    OUTPUT_LINE = 'LINE'

    DUMPFILE_DIR = 'dump_logs'


class Config:
    PRE_ARGUMENTS = {
        'verbose': {'default': 0, 'type': int, 'help': 'Verbose mode. [Level - 1:TRACE_ERROR, 2:INFO, 3:DEBUG]', 'choices': [0, 1, 2, 3]},
        'debug': {'default': False, 'help': '`--debug` is equivalent to `--verbose=3`.', 'action': 'store_true'},
        'log': {'default': None, 'type': str, 'help': 'Verbose log filename.', 'metavar': '{string}'}
    }

    ARGUMENTS = {
        'find': {'default': '', 'help': 'Find character string by regex and ignoring case. ex: "3\\d1", "HTTP"', 'metavar': '{string}'},
        'find_mode': {'default': 'REGEX', 'help': 'Find mode. [Mode name] or [1 - 4]\nREGEX, MATCH, BINARY, HEX', 'metavar': '[REGEX, MATCH, BINARY, HEX]'},
        'port': {'default': '', 'type': str, 'help': 'Filter port. It is source port or destination port. ex: =80, =53,80', 'metavar': '{int}'},
        'protocol': {'default': 'TCP,UDP', 'type': str, 'help': 'Filter Protocol. Default: "TCP,UDP"', 'metavar': '[ICMP, TCP, UDP]'},
        'ip': {'default': '', 'type': str, 'help': 'Filter IP. ex: =192.168.1.10, =192.168.1.10,192.168.1.20', 'metavar': '{string}'},
        'condition': {'default': '', 'help': 'Filter by detail condition. ex: "src_port=80;dest_port<=30000;ttl=64;flags=SYN,PSH"', 'metavar': '{string}'},
        'tracking': {'default': False, 'action': 'store_true', 'help': 'Tracking transfers that have been matched by filters.'},
        'stat_mode': {'default': 1, 'type': int, 'help': 'Statistics mode.\n0: None, 1: Captured transfers, 2: All transfers', 'choices': [0, 1, 2]},
        'stat_group': {'default': 0, 'type': int, 'help': 'Group the transfer in statistics.\n0: None, 1: Grouping by IPs and service port, 2: Grouping by IPs', 'choices': [0, 1, 2]},
        'output': {'default': 'TEXT', 'help': 'Output mode about header and data. [Mode name] or [0 - 7]\nNONE: none\nHEADER: header only, TEXT: text data\nBINARY: binary data, BINARY_ALL: binary headers and data\nHEX: hex data, HEX_ALL: hex headers and data\nLINE: single line', 'metavar': '[NONE, HEADER, TEXT, BINARY, BINARY_ALL, HEX, HEX_ALL, LINE]'},
        'output_raw': {'default': False, 'help': 'Output "Raw block". Show HEX data in each values.', 'action': 'store_true'},
        'dumpfile': {'default': 0, 'type': int, 'help': 'Dump data to files. Dir: `./dump_logs/`\n0: Off, 1: Dump data, 2: Dump headers and data', 'choices': [0, 1, 2]},
        'timeout': {'default': None, 'type': float, 'help': 'Stop automatically after the specified number of seconds.', 'metavar': '{float}'},

        'exclude_ssh': {'default': False, 'help': '`--exclude_ssh` is equivalent to `--condition="port!=22"`.', 'action': 'store_true'},
        'web_port': {'default': False, 'help': '`--web_port` is equivalent to `--port=80,443,53`.', 'action': 'store_true'},
        'general_port': {'default': False, 'help': '`--general_port` is equivalent to `--port=21,22,23,25,53,80,110,143,220,443,465,990,993,995,1433,3306`.', 'action': 'store_true'},

        'force': {'default': False, 'help': 'Run force if any filter options aren\'t specified.', 'action': 'store_true'},
        'version': {'default': False, 'help': 'Show version information.', 'action': 'store_true'}
    }

    CONDITION_RULES = {
        'port': {'type': int},
        'client_port': {'type': int},
        'src_port': {'type': int},
        'dest_port': {'type': int},
        'ttl': {'type': int},
        'flags': {'type': lambda v: v.upper(), 'list': True, 'types': lambda codes: TCPHeader.get_flags(codes), 'single': True},
        'seq': {'type': int},
        'ack': {'type': int},
        'window': {'type': int},
        'mss': {'type': int},
        'wscale': {'type': int},
        'sack': {'type': int}
    }



import base64
import codecs
import re
import sys


class DataInput:
    INPUT_RAW = 'NONE'
    INPUT_TEXT = 'TEXT'
    INPUT_BINARY = 'BINARY'
    INPUT_HEX = 'HEX'
    INPUT_BASE64 = 'BASE64'

    def __init__(self, mode=None):
        self._mode = mode

    def initialize(self, mode):
        self._mode = mode

    def get_data(self, input_data):
        if self._mode == self.INPUT_TEXT:
            input_data = input_data.replace("\\n", "\n")

            input_data = input_data.encode()
        elif self._mode == self.INPUT_BINARY:
            input_data = codecs.decode(input_data, 'unicode_escape').encode('latin1')
        elif self._mode == self.INPUT_HEX:
            input_data = self.decode_hex(input_data)
        elif self._mode == self.INPUT_BASE64:
            input_data = self.decode_base64(input_data)
        elif self._mode == self.INPUT_RAW or not self._mode:
            pass
        else:
            raise Exception('Unknown input mode.')

        return input_data

    def decode_hex(self, hex_data):
        hex_data = re.sub(r'\s', '', hex_data)

        try:
            if len(hex_data) % 2 != 0:
                raise Exception()

            byte_data = bytes.fromhex(hex_data)
        except Exception:
            raise Exception('Hex data decode error.')

        return byte_data

    def decode_base64(self, base64_data):
        try:
            base64_byte_data = base64_data.encode('utf-8')

            byte_data = base64.b64decode(base64_byte_data)
        except Exception:
            raise Exception('Base64 data decode error.')

        return byte_data


class DataOutput:
    OUTPUT_TEXT = 'TEXT'
    OUTPUT_BINARY = 'BINARY'
    OUTPUT_HEX = 'HEX'
    OUTPUT_BASE64 = 'BASE64'

    def __init__(self, mode=None):
        self._mode = mode

    def initialize(self, mode):
        self._mode = mode

    def get_data(self, data):
        v = None

        if self._mode == self.OUTPUT_TEXT:
            v = data.decode('utf-8', errors='ignore')
        elif self._mode == self.OUTPUT_BINARY:
            v = data
        elif self._mode == self.OUTPUT_HEX:
            v = self.get_hex_data(data)
        elif self._mode == self.OUTPUT_BASE64:
            v = self.get_base64_data(data)
        else:
            raise Exception('Unknown output mode.')

        return v

    def get_hex_data(self, data):
        data = ''.join(f'{byte:02x} ' for byte in data)

        return data

    def get_base64_data(self, data):
        base64_data = base64.b64encode(data)

        return base64_data.decode('utf-8')

    def output_binary(self, data):
        print(self.get_data(data), flush=True)


class InteractiveInput:
    def __init__(self, ctrlkey=False):
        self._ctrlkey = ctrlkey
        self._lines = ''

    def get_input(self):
        if not self._ctrlkey:
            input = self.get_line()
        else:
            input = self.get_stdin_read()

        return input

    def get_line(self):
        line = input()

        self._lines += line + '\n'

        lines = self._lines

        if not (re.search(r'(\r\n|\r|\n){2}$', lines) and lines.strip()):
            return None

        self._lines = ''

        return lines

    def get_stdin_read(self):
        return sys.stdin.read() + '\n'



import pytest

from ipsurv.requester.geoip import GeoIpRequester
from ipsurv.data_collector.geoip_collector import GeoIpCollector
from ipsurv.core.entity import Target, ValueData
from unittest.mock import MagicMock

import re
import os


@pytest.fixture
def args(mocker):
    args = mocker.Mock()
    args.fixed_format = '{ip}'
    args.fixed_format_params = ['ip']
    args.fixed_delimiter = ','
    args.alt_delimiter = ';'
    args.fixed_enclose = ''
    args.exhaustive = False

    return args


class TestGeoIpRequester:
    '''
    geoip2.database.Reader Mock.
    If you want to test the actual behavior, preparing the GeoIP data file and remove the setup method.
    '''
    @pytest.fixture(autouse=True)
    def setup(self, monkeypatch, mocker):
        if self._use_geoip_mock():
            monkeypatch.setattr(os.path, 'exists', lambda v: True)

            mock_geoip_reader = mocker.Mock(spec='geoip2.database.Reader')

            item = mocker.Mock()
            item.country.iso_code = 'US'
            item.city.continent.code = 'NA'
            item.autonomous_system_organization = 'GOOGLE'

            mock_geoip_reader.country = mocker.Mock(return_value=item)
            mock_geoip_reader.city = mocker.Mock(return_value=item)
            mock_geoip_reader.asn = mocker.Mock(return_value=item)

            monkeypatch.setattr(GeoIpRequester, '_get_geoip_reader', lambda *vs: mock_geoip_reader)
            monkeypatch.setattr(GeoIpRequester, '_import_geoip2', lambda *vs: mocker.Mock())

    def _use_geoip_mock(self):
        if os.getenv('GITHUB_ACTIONS') is not None or 'TOX_PACKAGE' in os.environ:
            return True

        # return True

        return False

    def test_request_methods(self):
        requester = GeoIpRequester()

        requester.initialize('/usr/local/share/GeoIP/')

        assert requester.get_data_path(None) == '/usr/local/share/GeoIP/'
        assert requester.get_data_path('/tmp/work') == '/tmp/work/'

        requester._delete_readers()

        assert requester.reader_country is None

        with pytest.raises(Exception):
            requester.get_reader('dummy')

        requester._delete_readers()
        assert requester.reader_country is None

    def test_import_geoip2(self):
        requester = GeoIpRequester()
        module = requester._import_geoip2()

        assert module is not None

    def test_errors(self, monkeypatch):
        requester = GeoIpRequester()
        requester.initialize('/usr/local/share/GeoIP/')
        requester.geoip2 = None

        with pytest.raises(Exception, match='GeoIp object is not'):
            requester.request('8.8.8.8')

        if self._use_geoip_mock():
            monkeypatch.setattr(os.path, 'exists', lambda v: False)

        requester = GeoIpRequester()
        requester.initialize('/tmp/')

        with pytest.raises(Exception, match='GeoIP data file none'):
            requester.request_country('8.8.8.8')

        with pytest.raises(Exception, match='GeoIP data file none'):
            requester.request_city('8.8.8.8')

        with pytest.raises(Exception, match='GeoIP data file none'):
            requester.request_asn('8.8.8.8')

    def test_request(self):
        requester = GeoIpRequester()
        requester.initialize('/usr/local/share/GeoIP/')

        response = requester.request('8.8.8.8')

        assert response['country'] == 'US'
        assert response['organization'] == 'GOOGLE'

    def test_request_country(self):
        requester = GeoIpRequester()
        requester.initialize('/usr/local/share/GeoIP/')

        response = requester.request_country('8.8.8.8')

        assert response['country'] == 'US'

    def test_request_city(self):
        requester = GeoIpRequester()
        requester.initialize('/usr/local/share/GeoIP/')

        response = requester.request_city('8.8.8.8')

        assert response['country'] == 'US'

    def test_request_asn(self):
        requester = GeoIpRequester()
        requester.initialize('/usr/local/share/GeoIP/')

        response = requester.request_asn('8.8.8.8')

        assert response['organization'] == 'GOOGLE'


class TestGeoIpCollector:
    @pytest.fixture
    def requester(self, mocker):
        requester = mocker.Mock(spec=GeoIpRequester)
        requester.request_country = MagicMock(return_value={'country': 'US'})
        requester.request_city = MagicMock(return_value={'continent': 'NA'})
        requester.request_asn = MagicMock(return_value={'organization': 'GOOGLE'})

        return requester

    def test_general(self, args, requester):
        collector = GeoIpCollector(requester, args)

        assert collector.get_name() == 'GEOIP'

    def test_request(self, args, requester):
        collector = GeoIpCollector(requester, args)

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, ['country'])

        assert success is True
        assert response['country'] == 'US'

        assert ('country' in collector.get_requires())

        success, response, response_time = collector.request(target, ['continent'])

        assert success is True
        assert response['continent'] == 'NA'

        target.ip = '8.8.8.8'

        success, response, response_time = collector.request(target, ['asn'])

        assert success is True
        assert response['organization'] == 'GOOGLE'

    def test_request_city(self, args, requester):
        collector = GeoIpCollector(requester, args)

        response = {'types': []}
        assert collector._request_city('8.8.8.8', response) is None

    def test_request_country(self, args, requester):
        collector = GeoIpCollector(requester, args)

        response = {'types': []}
        assert collector._request_country('8.8.8.8', response) is None

    def test_request_asn(self, args, requester):
        collector = GeoIpCollector(requester, args)

        response = {'types': []}
        assert collector._request_asn('8.8.8.8', response) is None

    #    def test_request_data(self, args):
#        pass
#
    def test_build_data(self, args, requester):
        collector = GeoIpCollector(requester, args)

        data = ValueData({})

        target = Target()
        target.identifier = '8.8.8.8'
        target.ip = '8.8.8.8'

        response = {
            'continent': None,
            'continent_name': None,
            'country': None,
            'country_name': None,
            'city': None,
            'city_name': None,
            'organization': None,
            'org': None,
            'timezone': None,
            'latitude': 0.1,
            'longitude': 0.1
        }

        collector.build_data(target, data, True, response, 10)

        assert data.get('geo') == '0.1,0.1'



from abc import ABC

from ipsurv.configure.args_builder import ArgsBuilder
from ipsurv.configs import Config
from ipsurv.core.entity import ValueDataFactory
from ipsurv.core.pipeline import Pipeline
from ipsurv.core.target_parser import TargetParser
from ipsurv.data_collector.basic_collectors import RdapCollector, DnsTxtCollector, IpInfoCollector, DnsReverseCollector
from ipsurv.data_collector.reactivity_collectors import ICMPCollector, TCPCollector, UDPCollector, HttpCollector
from ipsurv.data_collector.geoip_collector import GeoIpCollector
from ipsurv.data_collector.self_collector import SelfCollector
from ipsurv.requester.dns_resolver import DnsResolveRequester
from ipsurv.requester.http import HttpRequester
from ipsurv.requester.ip_info import IpInfoRequester
from ipsurv.requester.rdap import CountryDetector, RdapRequester
from ipsurv.requester.geoip import GeoIpRequester
from ipsurv.requester.server_reactivity import ServerReactivity
from ipsurv.serializer.json_serializer import JsonSerializer
from ipsurv.serializer.line_serializer import LineSerializer
from ipsurv.serializer.line_serializer import Serializer


class ObjectFactory(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#objectfactory
    """
    def get_config(self):
        """
        :rtype: Config
        """
        return Config

    def create_pipeline(self):
        """
        :rtype: Pipeline
        """
        return Pipeline()

    def create_value_data_factory(self, args, config):
        """
        :param args:
        :type args: argparse.Namespace
        :param config:
        :type config: Config
        :rtype: ValueDataFactory
        """
        return ValueDataFactory(config.MASTER_DATA, args.fixed_format_params)

    def create_args_builder(self, config, pipeline):
        """
        :param config:
        :type config: Config
        :param pipeline:
        :type pipeline: Pipeline
        :rtype: ArgsBuilder
        """
        return ArgsBuilder(config, pipeline)

    def create_target_parser(self, args, pipeline, dns_resolver):
        """
        :param args:
        :type args: argparse.Namespace
        :param pipeline:
        :type pipeline: Pipeline
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        :rtype: TargetParser
        """
        return TargetParser(args, pipeline, dns_resolver)

    def create_dns_resolver(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: DnsResolveRequester
        """
        return DnsResolveRequester(timeout=args.fixed_timeout['dns'])

    def create_collectors(self, args, dns_resolver):
        """
        :param args:
        :type args: argparse.Namespace
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        :rtype: dict
        """
        collectors = {}

        _collectors = args.fixed_collectors

        if 'rdap' in _collectors:
            collectors['rdap'] = self.create_rdap_collector(args)

        if 'dnstxt' in _collectors:
            collectors['dnstxt'] = self.create_dnstxt_collector(dns_resolver, args)

        if 'ipinfo' in _collectors:
            collectors['ipinfo'] = self.create_ipinfo_collector(args)

        if 'dnsreverse' in _collectors:
            collectors['dnsreverse'] = self.create_dns_reverse_collector(dns_resolver, args)

        if 'geoip' in _collectors:
            collectors['geoip'] = self.create_geoip_collector(args)

        return collectors

    def create_rdap_collector(self, args):
        """
        :param args:
        :type args: argparse.Namespace
       :rtype: RdapCollector
        """
        country_detector = CountryDetector()

        return RdapCollector(RdapRequester(country_detector, timeout=args.fixed_timeout['http']), args)

    def create_dnstxt_collector(self, dns_resolver, args):
        """
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        :param args:
        :type args: argparse.Namespace
        :rtype: DnsTxtCollector
        """
        return DnsTxtCollector(dns_resolver, args)

    def create_ipinfo_collector(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: IpInfoCollector
        """
        return IpInfoCollector(IpInfoRequester(timeout=args.fixed_timeout['http'], token=args.conf.get('ipinfo_token')), args)

    def create_geoip_collector(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: GeoIpCollector
        """
        return GeoIpCollector(GeoIpRequester(), args)

    def create_self_collector(self, args, dns_resolver, server_reactivity):
        """
        :param args:
        :type args: argparse.Namespace
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        :param server_reactivity:
        :type server_reactivity: ServerReactivity
        :rtype: SelfCollector
        """
        return SelfCollector(IpInfoRequester(timeout=args.fixed_timeout['http']), dns_resolver, server_reactivity, args)

    def create_dns_reverse_collector(self, dns_resolver, args):
        """
        :param dns_resolver:
        :type dns_resolver: DnsResolveRequester
        :param args:
        :type args: argparse.Namespace
        :rtype: SelfCollector
        """
        return DnsReverseCollector(dns_resolver, args)

    def create_reactivities(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: list
        """

        server_reactivities = []

        requester = self.create_server_reactivity(args)

        if args.icmp:
            server_reactivities.append(self.create_icmp_collector(requester, args))

        if args.tcp:
            server_reactivities.append(self.create_tcp_collector(requester, args))

        if args.udp:
            server_reactivities.append(self.create_udp_collector(requester, args))

        if args.http:
            http_requester = self.create_http(args)
            server_reactivities.append(self.create_http_collector(http_requester, args))

        return server_reactivities

    def create_server_reactivity(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: ServerReactivity
        """

        return ServerReactivity(timeout=args.fixed_timeout['reactivity'])

    def create_http(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: HttpRequester
        """
        return HttpRequester(timeout=args.fixed_timeout['reactivity'])

    def create_icmp_collector(self, requester, args):
        """
        :param server_reactivity:
        :type server_reactivity: ServerReactivity
        :param args:
        :type args: argparse.Namespace
        :rtype: ICMPCollector
        """
        return ICMPCollector(requester, args)

    def create_tcp_collector(self, requester, args):
        """
        :param server_reactivity:
        :type server_reactivity: ServerReactivity
        :param args:
        :type args: argparse.Namespace
        :rtype: TCPCollector
        """
        return TCPCollector(requester, args)

    def create_udp_collector(self, requester, args):
        """
        :param server_reactivity:
        :type server_reactivity: ServerReactivity
        :param args:
        :type args: argparse.Namespace
        :rtype: UDPCollector
        """
        return UDPCollector(requester, args)

    def create_http_collector(self, requester, args):
        """
        :param server_reactivity:
        :type server_reactivity: ServerReactivity
        :param args:
        :type args: argparse.Namespace
        :rtype: HttpCollector
        """
        return HttpCollector(requester, args)

    def create_serializer(self, args):
        """
        :param args:
        :type args: argparse.Namespace
        :rtype: Serializer
        """

        if not args.json:
            serializer = LineSerializer(args)
        else:
            serializer = JsonSerializer(args)

        return serializer



import threading
import socket


class DnsUtil:
    @classmethod
    def getaddrinfo(cls, hostname, port=None, timeout=8.0):
        thread = ResolveThread(hostname, port)

        cls._run(thread, timeout)

        return thread.sockaddr

    @classmethod
    def resolve(cls, host, port=None, timeout=None):
        sockaddr = DnsUtil.getaddrinfo(host, port, timeout)

        return sockaddr[0]

    @classmethod
    def reverse(cls, ip, timeout=8.0):
        thread = ReverseThread(ip)

        cls._run(thread, timeout)

        return thread.hostname

    @classmethod
    def _run(cls, thread, timeout):
        thread.start()

        thread.join(timeout)

        if thread.is_alive():
            raise socket.timeout('Socket timeout error.')
        elif thread.e:
            raise thread.e


class ResolveThread(threading.Thread):
    def __init__(self, hostname, port=None):
        super().__init__()

        self.hostname = hostname
        self.port = port

        self.sockaddr = None

        self.e = None
        self.daemon = True

    def run(self):
        try:
            sockaddrs = socket.getaddrinfo(self.hostname, self.port)

            self.sockaddr = sockaddrs[0][4]
        except socket.error as e:
            self.e = e


class ReverseThread(threading.Thread):
    def __init__(self, ip):
        super().__init__()

        self.ip = ip

        self.hostname = None

        self.e = None
        self.daemon = True

    def run(self):
        try:
            vals = socket.gethostbyaddr(self.ip)

            self.hostname = vals[0]
        except socket.error as e:
            self.e = e



import logging
from abc import ABC
import http.client


class Requester(ABC):
    """
    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#requester
    """

    def __init__(self, timeout=None):
        self.timeout = timeout
        self.host = None

    def get_host(self):
        return self.host

    def _create_http_connection(self, host, https=True):
        if https:
            conn = http.client.HTTPSConnection(host, timeout=self.timeout)
        else:
            conn = http.client.HTTPConnection(host, timeout=self.timeout)

        return conn

    def _http_exception(self, res, body):
        msg = 'Failure response.(Status:' + str(res.status) + ')'

        logging.log(logging.DEBUG, msg + '\n' + str(body, 'utf-8'))

        return http.client.HTTPException(msg)



import logging
import time

from ipsurv.configs import Constant
from ipsurv.core.pipeline import Pipeline
from ipsurv.core.target_groups import TargetGroups
from ipsurv.core.target_parser import TargetParser
from ipsurv.core.entity import HeaderTarget
from ipsurv.serializer.serializer import Serializer
from ipsurv.util.sys_util import AppException, System


class SurveyIps:
    def __init__(self, args, config, data_factory, target_parser, collectors, reactivities, pipeline, serializer):
        self.config = config
        self.data_factory = data_factory
        self.target_parser = target_parser  # type: TargetParser
        self.target_groups = TargetGroups(args, pipeline)

        self.collectors = collectors
        self.reactivities = reactivities

        self.pipeline = pipeline  # type: Pipeline
        self.serializer = serializer  # type: Serializer

    def initialize(self, args):
        for collector in self.collectors:
            collector.initialize(args)

        for reactivity in self.reactivities:
            reactivity.initialize(args)

    def dispatch(self, rows, args):
        if args.headers:
            self._output_headers(args)

        sequence = 0

        for row in rows:
            sequence += 1

            if (args.begin > sequence) or (args.end > 0 and args.end < sequence):
                continue

            self._survey_row(sequence, row, args)

    def _output_headers(self, args):
        headers = self.serializer.create_labels(args.fixed_format_params, args.headers)
        target = HeaderTarget()

        data = self.data_factory.build(headers)

        data.set_header(True)
        data.set('target', target)

        self.pipeline.pre_output_headers(data)

        row = self.pipeline.build(data, target)

        self.pipeline.output_result(row)

    def _survey_row(self, sequence, original, args):
        data = self.data_factory.create()

        data.set('sequence', sequence)
        data.set('original', original)

        try:
            original = self.pipeline.pre_target_parse(data, original)

            target = self.target_parser.parse(data, original, args)

            self._survey_target(data, target, args)

            row = self.pipeline.build(data, target)

            if not target.identifier_int:
                time.sleep(0.2)  # Buffer time
        except Exception as e:
            level = logging.ERROR if not isinstance(e, AppException) else logging.DEBUG

            logging.log(level, str(e), exc_info=True)

            row = self.pipeline.build_error(str(e))

        self.pipeline.output_result(row)

    def _survey_target(self, data, target, args):
        self.pipeline.pre_collect(data, target, args)

        skip = False

        if target.status == Constant.STATUS_EXIST:
            group = self.target_groups.find_group(data, target)

            requires = []

            if not group or not args.skip_duplicate:
                requires += self._survey_by_collectors(self.collectors, target, args, data, True)
            else:
                logging.info('SKIP:IP->' + target.identifier + ',GROUP->' + str(group.value))

                skip = True

            if not skip or args.skip_duplicate < 2:
                requires += self._survey_by_collectors(self.reactivities, target, args, data, False)

            success = self._finish(data, args, requires)
            data.set('success', success)

            if len(data.get('requests')) == 0:
                self.target_groups.put_group(data, target, args.group, None)

        self.pipeline.post_collect(data, target, args, skip)

        if System.is_logging():
            System.output_data('COLLECTED_DATA', data.get_values())
            System.output_data('REQUESTS', data.get('requests'), logging.DEBUG)

    def _survey_by_collectors(self, collectors, target, args, data, is_source):
        requires = []

        for collector in collectors:
            reqs = self._survey_by_collector(collector, target, args, data, is_source)

            requires += reqs

        return requires

    def _survey_by_collector(self, collector, target, args, data, is_source):
        name = collector.get_name()

        reqs = list(set(collector.get_requires()) & set(args.fixed_format_params))

        self.pipeline.pre_request(data, name, collector)

        requires = self._require_request(data, reqs)

        if len(requires) > 0 or len(reqs) == 0 or args.all_collect:
            requires = requires if not args.all_collect else None

            success, response, response_time = collector.request(target, requires)

            self.pipeline.post_request(data, name, collector, success, response)

            if is_source:
                self._update_group(data, collector, target, args, response)

            collector.build_data(target, data, success, response, response_time)

            data.update('requests', lambda v: v + [name])
        else:
            logging.log(logging.DEBUG, 'UNNECESSARY:' + name)

        return reqs

    def _require_request(self, data, keys):
        requires = []

        for param in data.get_data().keys():
            v = data.get(param)

            if ((v is None or v is False) and param in keys):
                requires.append(param)

        return requires

    def _finish(self, data, args, requires):
        success = not self._require_request(data, args.fixed_format_params)

        for k, v in data.get_data().items():
            if v is None:
                if k in requires:
                    v = ''
                elif k not in self.config.FORMAT_PARAMS:
                    v = '-'

                data.set(k, v)

        return success

    def _update_group(self, data, collector, target, args, response):
        if not data.get('group_int'):
            cidr = collector.get_cidr(response)

            group = self.target_groups.put_group(data, target, args.group, cidr)

            if group:
                logging.info('GROUP:' + str(group.value))



import pytest

from ipsurv.util.args_util import ArgsHelper, StdinLoader
import re
import io


class TestArgValidator:
    def test_init_logging(self):
        assert ArgsHelper.init_logging(1, None) is None
        assert ArgsHelper.init_logging(2, None) is None


class TestStdinLoader:
    def test_read_stdin(self, monkeypatch):
        monkeypatch.setattr('select.select', lambda a, b, c, d: ("1\n4\n", None, None))
        monkeypatch.setattr('sys.stdin', io.StringIO("1\n4\n"))
        monkeypatch.setattr('sys.stdin.fileno', lambda: 0)

        lines = StdinLoader.read_stdin()

        assert len(lines) == 2

    def test_load_env(self, monkeypatch):
        monkeypatch.setenv("TEST1", '{"abc": 1}')

        env = StdinLoader.load_env('TEST1')

        assert env['abc'] == 1

        monkeypatch.setenv("TEST1", '{"abc": 1')

        env = StdinLoader.load_env('TEST1')

        assert len(env) == 0



import logging
import time
from abc import ABC, abstractmethod
from ipsurv.core.entity import Target, ValueData
from ipsurv.util.sys_util import System


class DataCollector(ABC):
    """
    :param requester:
    :type requester: ipsurv.requester.requester.Requester
    :param args:
    :type args: argparse.Namespace

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#datacollector
    """

    def __init__(self, requester, args):
        self.requester = requester

    def initialize(self, args):
        pass

    @abstractmethod
    def get_name(self):  # pragma: no cover
        return ''

    def request(self, target, requires):
        # type: (Target, list) -> tuple
        """
        :param target:
        :type target: Target
        :param requires:
        :type requires: list
        """

        name = self.get_name()

        logging.info('REQUEST ' + name + '...')

        begin_time = time.time()

        success = False
        response = {}

        try:
            (success, response) = self.request_data(target, requires)
        except Exception as e:
            error_name = name + ' ERROR'
            error = str(e)
            response['error'] = error

            if System.get_log_level() == logging.INFO:
                logging.log(logging.INFO, error_name + ':' + error)
            else:
                logging.log(logging.DEBUG, error_name, exc_info=True)

        response_time = self._get_measure_time(begin_time)

        if System.is_logging():
            System.output_data(name + '_DATA', response, logging.DEBUG)

            logging.log(logging.INFO, name + ':' + ('OK' if success else 'NG'))
            logging.log(logging.INFO, name + '_TIME(ms):' + str(response_time))

        return success, response, response_time

    @abstractmethod
    def request_data(self, target, requires):  # pragma: no cover
        # type: (Target, list) -> tuple
        """
       :param target:
       :type target: Target
       :param requires:
       :type requires: list
       """

        return None, None

    @abstractmethod
    def get_requires(self):  # pragma: no cover
        return []

    def get_cidr(self, response):
        return None

    @abstractmethod
    def build_data(self, target, data, success, response, response_time):  # pragma: no cover
        # type: (Target, ValueData, bool, dict, float) -> None
        """
       :param target:
       :type target: Target
       :param data:
       :type data: ValueData
       :param response:
       :type response: dict
       :param response_time:
       :type response_time: float
       """

        pass

    def _append_error(self, data, response):
        error = response.get('error')

        if error:
            data.get('errors').append(error)

    def put(self, data, response, key, key2=None):
        key2 = key if key2 is None else key2

        v = response[key] if key in response else None

        data.set(key2, v)

    def fill(self, data, response, key, key2=None):
        """
       :param data:
       :type data: ValueData
       :param response:
       :type response: dict
       :param key:
       :type key: str
       :param key2:
       :type key2: str
       """

        key2 = key if key2 is None else key2

        if not data.get(key2):
            v = response[key] if key in response else None

            data.set(key2, v)

    def _get_measure_time(self, begin_time, digit=1):
        end_time = time.time()

        response_time = end_time - begin_time

        response_time = round(response_time * 1000, digit)

        return response_time



from ipsurv.data_collector.data_collector import DataCollector
from ipsurv.requester.requester import Requester
from ipsurv.core.entity import Target


class PassDataCollector(DataCollector):
    """
    :param name:
    :type name: str
    :param requester:
    :type requester: PassRequester
    :param args:
    :type args: argparse.Namespace
    """

    def __init__(self, name, requester, args):
        super().__init__(requester, args)

        self._name = name

    def get_name(self):
        return self._name.upper()

    def request_data(self, target, requires):
        # type: (Target, list) -> tuple
        return self.requester.request(target)

    def get_requires(self):
        return []

    def build_data(self, target, data, success, response, response_time):
        data.set(self._name.lower() + '_time', response_time)

        for k, v in response.items():
            self.put(data, response, k)

        self._append_error(data, response)


class PassRequester(Requester):
    def request(self, target):
        # type : Target
        """
        :param target:
        :type target: Target
        """

        return False, {}



import bisect
import ipaddress
from ipsurv.core.pipeline import Pipeline
from ipsurv.core.entity import TargetGroup


class TargetGroups:
    def __init__(self, args, pipeline):
        self.groups = []
        self.group_indexes = []

        self.pipeline = pipeline  # type: Pipeline

        self.ignore = True if args.group is None and not args.skip_duplicate else False

    def find_group(self, data, target):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: ipsurv.entity.Target
        :rtype: TargetGroup
        """
        if self.ignore:
            return None

        group = self.pipeline.find_group(data, target)

        if group is None:
            identifier_int = self.pipeline.get_group_identify(data, target)

            group = self._find_indexes(identifier_int)

        if group is not None:
            data.set('group', group.value)
            data.set('group_int', group.begin_int)
            data.set('group_found', True)

        return group

    def put_group(self, data, target, group_type, cidr):
        """
        :param data:
        :type data: ValueData
        :param target:
        :type target: ipsurv.entity.Target
        :param group_type:
        :type group_type: int
        :param cidr:
        :type cidr: str
        :rtype: TargetGroup
        """
        if self.ignore:
            return None

        group = self.pipeline.create_group(data, target, group_type, cidr)

        if group is None:
            group = self._create_group_by_identifier(target, group_type, cidr)

        if group is not None:
            data.set('group', group.value)
            data.set('group_int', group.begin_int)
            data.set('group_found', True)

            self._add_indexes(group)

        return group

    def _create_group_by_identifier(self, target, group_type, cidr):
        end_int = None

        if group_type == 'network':
            if cidr is None:
                return None

            begin_int, end_int = self._get_network_range(cidr)
        elif group_type is None:
            begin_int = end_int = target.identifier_int
        else:
            begin_int, end_int = self._get_network_range(target.identifier + '/' + str(group_type))

        group = None

        if begin_int:
            value = str(ipaddress.ip_address(begin_int))
            group = TargetGroup(begin_int, end_int, value)

        return group

    def _get_network_range(self, cidr):
        network = ipaddress.ip_network(cidr, strict=False)

        first_ip = network.network_address + 1
        last_ip = network.broadcast_address - 1

        return int(first_ip), int(last_ip)

    def _find_indexes(self, identifier_int):
        found = None

        index = bisect.bisect_left(self.group_indexes, identifier_int)

        if index >= 0 and len(self.groups) > index:
            bucket = self.groups[index]

            (begin_int, end_int, group) = bucket

            if end_int is not None:
                if begin_int <= identifier_int and identifier_int <= end_int:
                    found = bucket
            else:
                if begin_int == identifier_int:
                    found = bucket

        group = None

        if found:
            (begin_int, end_int, value) = found

            group = TargetGroup(begin_int, end_int, value)

        return group

    def _add_indexes(self, group):
        group_int = group.end_int if group.end_int is not None else group.begin_int

        index = bisect.bisect_left(self.group_indexes, group_int)

        bucket = (group.begin_int, group.end_int, group.value)

        self.groups.insert(index, bucket)
        self.group_indexes.insert(index, group_int)



import random

from ipsurv.data_collector.data_collector import DataCollector
from ipsurv.util.network_util import DnsUtil


class SelfCollector(DataCollector):
    """
    :param requester:
    :type requester: ipsurv.requester.ip_info.IpInfoRequester
    :param dns_resolver:
    :type dns_resolver: ipsurv.requester.dns_resolver.DnsResolveRequester
    :param server_reactivity:
    :type server_reactivity: ipsurv.requester.server_reactivity.ServerReactivity
    :param args:
    :type args: argparse.Namespace
    """
    def __init__(self, requester, dns_resolver, server_reactivity, args):
        super().__init__(requester, args)

        self.dns_resolver = dns_resolver
        self.server_reactivity = server_reactivity
        self.dns_timeout = args.fixed_timeout['dns']

    def get_name(self):
        return 'SELF_INFO'

    def get_requires(self):
        return []

    def request_data(self, target, requires):
        DnsUtil.resolve(self.requester.get_host(), timeout=self.dns_timeout)

        success, response = self.requester.request(None)

        resolver = self.dns_resolver.get_resolver()

        response['local_dns'] = resolver.nameservers

        ip = random.choice(['8.8.8.8', '8.8.4.4', '1.1.1.1'])

        response['local_ip'] = self.server_reactivity.request_local_ip(ip=ip)

        return success, response

    def build_data(self, target, data, success, response, response_time):
        self.put(data, response, 'ip')
        self.put(data, response, 'hostname')
        self.put(data, response, 'country')
        self.put(data, response, 'city', 'city_name')
        self.put(data, response, 'region', 'region_name')
        self.put(data, response, 'postal')
        self.put(data, response, 'loc', 'geo')
        self.put(data, response, 'org', 'organization')
        self.put(data, response, 'timezone')
        self.put(data, response, 'local_dns')
        self.put(data, response, 'local_ip')

        self._append_error(data, response)



import sys

import logging
import signal
import threading
from datetime import datetime
from functools import partial

from ipscap.configs import Config, Constant
from ipscap.core.object_factory import ObjectFactory
from ipscap.core.pipeline import Pipeline
from ipscap.service.dumpfile import DumpFile
from ipscap.service.eth_socket import EthSocket
from ipscap.service.packet_filter import PacketFilter
from ipscap.service.protocol_service import ProtocolService
from ipscap.service.transfer_store import TransferStore
from ipscap.service.view_helper import ViewHelper
from ipscap.util.raw_socket_parser import IPHeaderParser
from ipsurv.util.sys_util import AppException
import time


class IpsCapCmd:
    def __init__(self, factory):
        self.factory = factory  # type: ObjectFactory
        self.pipeline = self.factory.create_pipeline()  # type: Pipeline
        self.config = self.factory.get_config()  # type: Config

        self.capture_thread = None  # type: CaptureThread
        self.transfer_store = None  # type: TransferStore
        self.dumpfile = None  # type: DumpFile
        self.begin_tm = None
        self.view_helper = None  # type: ViewHelper

    def run(self):
        try:
            self._pre_initialize()

            args, parser, ev_parser = self._parse_args()

            self._initialize(ev_parser, args)

            self._verify_args(args, parser)

            self.dispatch(args)
        except Exception as e:
            self.view_helper.output_error(e)

            logging.log(logging.ERROR, str(e), exc_info=True)

    def _verify_args(self, args, parser):
        if args.version:
            self.view_helper.show_version()
        elif len(sys.argv) == 1:
            parser.print_help()
            sys.exit()
        elif not args.has_filters and not args.force:
            self.view_helper.show_nofilters()

    def _pre_initialize(self):
        self.view_helper = self.factory.create_view_helper()

    def _initialize(self, ev_parser, args):
        self._register_protocol_services()

        packet_filter = self.factory.create_packet_filter(ev_parser)
        packet_filter.initialize(args)

        self.transfer_store = self.factory.create_transfer_store()

        self.dumpfile = self.factory.create_dumpfile(self.pipeline)

        if args.dumpfile:
            self.dumpfile.initialize(Constant.DUMPFILE_DIR)

        signal.signal(signal.SIGINT, partial(self.signal_stop, args=args))

        self.begin_tm = time.time()

        self.capture_thread = CaptureThread(self.factory, self.pipeline, packet_filter, self.transfer_store, self.dumpfile, self.view_helper, args)

        self.capture_thread.initialize()

        self.pipeline.initialize(self.config)

    def _register_protocol_services(self):
        icmp_service = self.factory.create_icmp_protocol_service()
        tcp_service = self.factory.create_tcp_protocol_service()
        udp_service = self.factory.create_udp_protocol_service()

        ProtocolService.register_services(icmp_service, tcp_service, udp_service)

    def _parse_args(self):
        args_builder = self.factory.create_args_builder(self.config, self.pipeline)

        return args_builder.parse()

    def dispatch(self, args):
        self.view_helper.show_head(args)

        self.capture_thread.start()

        self.capture_thread.join(args.timeout)

        self.capture_thread.trigger_stop()

        self._complete(args)

    def signal_stop(self, sig, frame, args):
        self.capture_thread.trigger_stop()

        self.view_helper.show_stopped()

        self._complete(args)

        sys.exit()

    def _complete(self, args):
        transfers = self.transfer_store.totalize(args.stat_group)

        self.pipeline.complete(transfers)

        end_tm = time.time()

        self.view_helper.show_statistics(transfers, self.begin_tm, end_tm, args)

        if args.dumpfile:
            self.view_helper.show_dumpfile_info(self.dumpfile)


class CaptureThread(threading.Thread):
    def __init__(self, factory, pipeline, packet_filter, transfer_store, dumpfile, view_helper, args):
        super().__init__()

        self.factory = factory  # type: ObjectFactory
        self.pipeline = pipeline  # type: Pipeline
        self.packet_filter = packet_filter  # type: PacketFilter
        self.transfer_store = transfer_store  # type: TransferStore
        self.dumpfile = dumpfile  # type: DumpFile
        self.view_helper = view_helper  # type: ViewHelper
        self.args = args

        self.eth_socket = None  # type: EthSocket
        self.ip_header_parser = None  # type: IPHeaderParser

        self._active = True

    def initialize(self):
        self.eth_socket = self.factory.create_eth_socket()

        self.eth_socket.create_socket()

        eth_ips = self.eth_socket.get_eth_ips_int()

        self.pipeline.pass_eth_ips(eth_ips)

        logging.log(logging.INFO, 'ETH_IPS: ' + str(eth_ips) + '\n')

        self.ip_header_parser = self.factory.create_ip_header_parser()

        self.ip_header_parser.initialize(eth_ips)

    def trigger_stop(self):
        self._active = False

    def run(self):
        self.pipeline.pre_recieve_loop(self.eth_socket, self.ip_header_parser)

        while True:
            try:
                raw_data = self.eth_socket.recvfrom(Constant.RECV_BUF_SIZE)

                eth_header = self.eth_socket.get_eth_header(raw_data)

                self.pipeline.pass_eth_header(raw_data, eth_header)

                if not self.eth_socket.is_enabled_protocol(eth_header):
                    self.view_helper.output_not_support(eth_header)
                    continue

                ip_mtu = self.eth_socket.get_ip_mtu(raw_data)

                (ip_header, protocol_header) = self._parse_ip_mtu(ip_mtu)

                is_capture = self._verify_capture(ip_header, protocol_header)

                passage_num = -1

                if (is_capture and self.args.stat_mode == 1) or self.args.stat_mode == 2:
                    (passage_num) = self.transfer_store.add(ip_header, protocol_header)

                if is_capture:
                    self._process_captured_transfer(ip_header, protocol_header, passage_num)
            except Exception as e:
                self.view_helper.output_error(e)

                logging.log(logging.ERROR, str(e), exc_info=True)

            if not self._active:
                break

    def _parse_ip_mtu(self, mtu_data):
        try:
            ip_header = self.ip_header_parser.parse(mtu_data, self.args.output_raw)

            self.pipeline.pass_ip_header(mtu_data, ip_header)

            header_parser = self.ip_header_parser.get_header_parser(ip_header)

            header_parser = self.pipeline.pass_header_parser(ip_header, header_parser)

            protocol_header = header_parser.parse(ip_header, mtu_data, self.args.output_raw)
        except Exception:
            raise AppException('Packet parse error.\n' + self.view_helper.get_hex_data(mtu_data))

        return (ip_header, protocol_header)

    def _verify_capture(self, ip_header, protocol_header):
        is_capture = self.packet_filter.verify_capture(ip_header, protocol_header, self.args)

        return self.pipeline.verify_capture(self.packet_filter, ip_header, protocol_header, is_capture)

    def _process_captured_transfer(self, ip_header, protocol_header, passage_num):
        self.view_helper.output_debug(ip_header, protocol_header)

        if self.args.dumpfile:  # Dump logs
            append_header = (self.args.dumpfile == 2)

            if len(protocol_header.get_sanitized_data()) > 0 or append_header:
                self.dumpfile.write(ip_header, protocol_header, append_header)

        self.pipeline.process_captured_transfer(ip_header, protocol_header, passage_num)

        protocol_service = ProtocolService.get_service(ip_header.protocol)  # type: ProtocolService

        protocol_service.show_transfer(ip_header, protocol_header, passage_num, datetime.now(), self.args)



import json
import logging
import re
from urllib.parse import urlparse

from ipsurv.requester.requester import Requester


class CountryDetector:
    """
    Detecting country code by address.
    """

    COUNTRIES = {
        'afghanistan': 'AF', 'albania': 'AL', 'algeria': 'DZ', 'angola': 'AO', 'argentina': 'AR', 'armenia': 'AM', 'australia': 'AU', 'austria': 'AT',
        'azerbaijan': 'AZ', 'bahrain': 'BH', 'bangladesh': 'BD', 'belarus': 'BY', 'belgium': 'BE', 'benin': 'BJ', 'bhutan': 'BT', 'bolivia': 'BO',
        'bosnia and herzegovina': 'BA', 'botswana': 'BW', 'brazil': 'BR', 'bulgaria': 'BG', 'burkina faso': 'BF', 'burundi': 'BI', 'cambodia': 'KH', 'cameroon': 'CM',
        'canada': 'CA', 'chad': 'TD', 'chile': 'CL', 'colombia': 'CO', 'congo': 'CG',
        'costa rica': 'CR', 'croatia': 'HR', 'cuba': 'CU', 'cyprus': 'CY', 'czech': 'CZ', "côte d'ivoire": 'CI', 'denmark': 'DK', 'djibouti': 'DJ', 'dominica': 'DM',
        'ecuador': 'EC', 'egypt': 'EG', 'el salvador': 'SV', 'equatorial guinea': 'GQ', 'eritrea': 'ER', 'estonia': 'EE', 'ethiopia': 'ET',
        'fiji': 'FJ', 'finland': 'FI', 'france': 'FR', 'gabon': 'GA', 'gambia': 'GM', 'georgia': 'GE', 'germany': 'DE', 'ghana': 'GH', 'greece': 'GR', 'greenland': 'GL',
        'guam': 'GU', 'guatemala': 'GT', 'guinea': 'GN', 'guinea-bissau': 'GW', 'guyana': 'GY', 'haiti': 'HT', 'honduras': 'HN', 'hong kong': 'HK', 'hungary': 'HU',
        'iceland': 'IS', 'india': 'IN', 'indonesia': 'ID', 'iran': 'IR', 'iraq': 'IQ', 'ireland': 'IE', 'israel': 'IL', 'italy': 'IT', 'jamaica': 'JM', 'japan': 'JP',
        'jordan': 'JO', 'kazakhstan': 'KZ', 'kenya': 'KE', 'korea': 'KR', 'south korea': 'KR', 'kuwait': 'KW',
        'kyrgyzstan': 'KG', 'latvia': 'LV', 'lebanon': 'LB', 'lesotho': 'LS', 'liberia': 'LR', 'libya': 'LY', 'lithuania': 'LT',
        'macao': 'MO', 'madagascar': 'MG', 'malawi': 'MW', 'malaysia': 'MY', 'maldives': 'MV', 'mali': 'ML',
        'malta': 'MT', 'mauritania': 'MR', 'mauritius': 'MU', 'mexico': 'MX', 'micronesia': 'FM', 'moldova': 'MD', 'monaco': 'MC',
        'mongolia': 'MN', 'montenegro': 'ME', 'morocco': 'MA', 'mozambique': 'MZ', 'myanmar': 'MM', 'namibia': 'NA', 'nepal': 'NP', 'netherlands': 'NL',
        'new zealand': 'NZ', 'nicaragua': 'NI', 'niger': 'NE', 'nigeria': 'NG', 'norway': 'NO', 'oman': 'OM', 'pakistan': 'PK', 'palestine': 'PS', 'panama': 'PA',
        'papua new guinea': 'PG', 'paraguay': 'PY', 'peru': 'PE', 'philippines': 'PH', 'poland': 'PL', 'portugal': 'PT', 'puerto rico': 'PR', 'qatar': 'QA', 'romania': 'RO',
        'russia': 'RU', 'rwanda': 'RW', 'saudi arabia': 'SA', 'senegal': 'SN', 'serbia': 'RS', 'sierra leone': 'SL', 'singapore': 'SG', 'slovakia': 'SK', 'slovenia': 'SI',
        'solomon islands': 'SB', 'somalia': 'SO', 'south africa': 'ZA', 'south sudan': 'SS', 'spain': 'ES', 'sri lanka': 'LK', 'sudan': 'SD', 'swaziland': 'SZ',
        'sweden': 'SE', 'switzerland': 'CH', 'swiss confederation': 'CH', 'syria': 'SY', 'taiwan': 'TW', 'tajikistan': 'TJ', 'tanzania': 'TZ',
        'thailand': 'TH', 'togo': 'TG', 'tunisia': 'TN', 'turkey': 'TR', 'turkmenistan': 'TM', 'uganda': 'UG', 'ukraine': 'UA',
        'uruguay': 'UY', 'uzbekistan': 'UZ', 'venezuela': 'VE',
        'viet nam': 'VN', 'yemen': 'YE', 'zambia': 'ZM', 'zimbabwe': 'ZW',

        'china': 'CN', 'pr china': 'CN',
        'united states': 'US', 'usa': 'US', 'u.s.a': 'US', 'u.s': 'US', 'united states of america': 'US',
        'united kingdom': 'GB', 'england': 'GB', 'great britain': 'GB', 'u.k': 'GB',
        'united arab emirates': 'AE', 'uae': 'AE'
    }

    COUNTRIES_REGEX = {
        'BO': r'bolivia',
        'CF': r'central african',
        'CD': r"(republic\s+[\w\s',.-]{5,10}\s+congo|congo[,\s]+[\w\s',.-]{10,20}\s+republic)",
        'CN': r"(republic\s+[\w\s',.-]{2,5}\s+china|china[,\s]+[\w\s',.-]{2,5}\s+republic)",
        'CZ': r'czech',
        'DO': r'dominican',
        'FM': r'micronesia',
        'HK': r'hksar',
        'IR': r'iran',
        'KP': r"(democratic\s+[\w\s',.-]{2,25}\s+korea|korea[,\s]+[\w\s',.-]{2,25}\s+democratic)",
        'KR': r"(republic\s+[\w\s',.-]{2,5}\s+korea|korea[,\s]+[\w\s',.-]{2,5}\s+republic)",
        'LA': r"lao\s+[\w\s',.-]{2,10}\s+democratic",
        'MD': r'moldova',
        'PS': r'palestine',
        'RU': r'(russian|russia)',
        'SY': r'syrian',
        'TW': r'taiwan',
        'TZ': r'tanzania',
        'US': r'(usa|united states)',
        'VE': r'(venezuela|bolivarian)'
    }

    def detect_by_address(self, v):
        v = re.sub(r'^[\s,/]+|[\s,/]+$', '', v)
        v = re.sub(r'[ \t]+', ' ', v)

        lines = re.split(r'[\r\n,;]', v)

        code = self.detect_by_country(lines[-1])

        if code is None:
            code = self.detect_by_country(lines[0])

        return code

    def detect_by_country(self, word):
        word = word.strip(' .')
        v = word.lower()

        code = None

        if v in self.COUNTRIES:
            code = self.COUNTRIES[v]
        else:
            c = word.upper()

            codes = list(self.COUNTRIES.values())

            if c in codes:
                code = c

        if code is None:
            for cd, regex in self.COUNTRIES_REGEX.items():
                if re.search(r'(^|[\s,])' + regex + r'($|[\s,])', v, flags=re.I):
                    code = cd
                    break

        return code


class RdapRequester(Requester):
    """
    :param country_detector:
    :type country_detector: CountryDetector
    :param fill:
    :type fill: bool
    :param timeout:
    :type timeout: float

    Description:
    https://deer-hunt.github.io/ipsurv/pages/ipsurv-cmd/program_architecture_classes.html#requester
    """

    ID_ICANN = 0
    ID_ARIN = 1
    ID_APNIC = 2
    ID_RIPE = 3
    ID_LACNIC = 4
    ID_AFRINIC = 5

    RDAP_ICANN = 'https://rdap.iana.org/'
    RDAP_ARIN = 'https://rdap.arin.net/registry/'
    RDAP_APNIC = 'https://rdap.apnic.net/'
    RDAP_RIPE = 'https://rdap.db.ripe.net/'
    RDAP_LACNIC = 'https://rdap.lacnic.net/rdap/'
    RDAP_AFRINIC = 'https://rdap.afrinic.net/rdap/'

    RDAP_SERVERS = {
        0: RDAP_ICANN,
        1: RDAP_ARIN,
        2: RDAP_APNIC,
        3: RDAP_RIPE,
        4: RDAP_LACNIC,
        5: RDAP_AFRINIC
    }

    COLLATIONS = {
        1: 2, 2: 3, 5: 3, 14: 2, 25: 3, 27: 2, 31: 3, 36: 2, 37: 3, 39: 2, 41: 5, 42: 2,
        43: 2, 46: 3, 49: 2, 51: 3, 58: 2, 59: 2, 60: 2, 61: 2, 62: 3, 77: 3, 78: 3, 79: 3,
        80: 3, 81: 3, 82: 3, 83: 3, 84: 3, 85: 3, 86: 3, 87: 3, 88: 3, 89: 3, 90: 3, 91: 3,
        92: 3, 93: 3, 94: 3, 95: 3, 101: 2, 102: 5, 103: 2, 105: 5, 106: 2, 109: 3, 110: 2,
        111: 2, 112: 2, 113: 2, 114: 2, 115: 2, 116: 2, 117: 2, 118: 2, 119: 2, 120: 2,
        121: 2, 122: 2, 123: 2, 124: 2, 125: 2, 126: 2, 133: 2, 141: 3, 145: 3, 150: 2,
        151: 3, 153: 2, 154: 5, 163: 2, 171: 2, 175: 2, 176: 3, 177: 4, 178: 3, 179: 4,
        180: 2, 181: 4, 182: 2, 183: 2, 185: 3, 186: 4, 187: 4, 188: 3, 189: 4, 190: 4,
        191: 4, 193: 3, 194: 3, 195: 3, 196: 5, 197: 5, 200: 4, 201: 4, 202: 2, 203: 2,
        210: 2, 211: 2, 212: 3, 213: 3, 217: 3, 218: 2, 219: 2, 220: 2, 221: 2, 222: 2,
        223: 2
    }

    def __init__(self, country_detector, fill=True, timeout=None):
        super().__init__(timeout)

        self.host = 'rdap.arin.net'
        self.country_detector = country_detector  # type: CountryDetector
        self.fill = fill

    def detect_server_from_ip(self, ip):
        serial1 = self.get_octet1_by_ip(ip)

        server = None

        if serial1 is not None:
            if serial1 in self.COLLATIONS:
                server_id = self.COLLATIONS[serial1]

                server = self.RDAP_SERVERS[server_id]

        if server is None:
            server = self.RDAP_ARIN

        return server

    def get_octet1_by_ip(self, ip):
        match = re.search(r'^(\d{1,3})\.', ip)

        v = None

        if match is not None:
            v = int(match.group(1))

        return v

    def get_octet2_by_ip(self, ip):
        match = re.search(r'^(\d{1,3})\.(\d{1,3})', ip)

        v = None

        if match is not None:
            v = int(match.group(1)) * 1000 + int(match.group(2))

        return v

    def get_id_from_server(self, server):
        server_id = None

        for k, v in self.RDAP_SERVERS.items():
            if v == server:
                server_id = k
                break

        return server_id

    def request(self, ip, url=None):
        if url is None:
            url = self.detect_server_from_ip(ip)

        res, body = self.request_ip(url, ip)

        success = False
        response = {}

        if res.status == 200:
            response = json.loads(body)

            if self.fill:
                response = self._fill_data(response)

            success = True
        else:
            raise self._http_exception(res, body)

        return success, response

    def _fill_data(self, response):
        cidr = None

        cidrs = response.get('cidr0_cidrs')

        if cidrs:
            if len(cidrs) > 0:
                val = cidrs[0]

                cidr = val['v4prefix'] + '/' + str(val['length'])

        response['cidr'] = cidr

        primary_entity = None

        for entity in response['entities']:
            if not entity.get('vcardArray') or len(entity['vcardArray']) < 2:
                continue

            vcards = entity['vcardArray'][1]

            if vcards is None:
                continue

            entity_card = {}

            entity_card['handle'] = entity.get('handle')
            entity_card['roles'] = entity.get('roles')

            for vcard in vcards:
                if len(vcard) > 0:
                    if vcard[0] == 'fn':
                        entity_card['name'] = vcard[3]

                    if vcard[0] == 'adr':
                        entity_card['address'] = vcard[1]['label']

                        primary_entity = entity_card

                        break

            if primary_entity:
                break

        response['primary_entity'] = primary_entity

        country_updated = False

        if not response.get('country') and primary_entity:
            address = primary_entity['address'] if primary_entity['address'] is not None else None
            response['country'] = self.country_detector.detect_by_address(address)
            country_updated = True

        response['country_updated'] = country_updated

        return response

    def request_ip(self, url, ip=None):
        if ip is not None:
            url = url + 'ip/' + ip

        logging.info('RDAP_URL:' + url)

        res, body = self.request_http(url)

        return res, body

    def request_http(self, url, n=1, max_redirect=5):
        if n >= max_redirect:
            return None, None

        parsed_url = urlparse(url)
        host = parsed_url.hostname

        conn = self._create_http_connection(host)

        res = None
        body = None

        redirect = False

        try:
            conn.request('GET', parsed_url.path)

            res = conn.getresponse()

            if res.status == 200:
                body = res.read()
            elif res.status in (301, 302, 303, 307, 308):
                redirect = True
            else:
                body = res.read()
        except Exception as e:
            raise e
        finally:
            conn.close()

        if redirect:
            redirect_url = res.getheader('Location')

            res, body = self.request_http(redirect_url, n + 1)

        return res, body



import pytest

from ipsurv.core.target_groups import TargetGroups
from ipsurv.core.entity import TargetGroup
from ipsurv.core.pipeline import Pipeline
from ipsurv.core.entity import ValueData, Target


class TestTargetGroups:
    @pytest.fixture(autouse=True)
    def setup(self, args):

        pipeline = Pipeline()

        self.target_groups = TargetGroups(args, pipeline)

    @pytest.fixture
    def args(self, mocker):
        args = mocker.Mock()
        args.fixed_format_params = []
        args.fixed_timeout = {'dns': 0, 'http': 0, 'reactivity': 0}
        args.fixed_collectors = []
        args.autodetect = False

        return args

    def test_put_group(self, args):
        data = ValueData({})

        target = Target()
        target.ip = '192.168.1.10'
        target.identifier = '192.168.1.10'

        group_type = '8'

        pipeline = Pipeline()

        targetGroups = TargetGroups(args, pipeline)
        group = targetGroups.put_group(data, target, group_type, None)

        assert group.value == '192.0.0.1'
        assert data.get('group') == '192.0.0.1'
        assert group.begin_int == 3221225473
        assert data.get('group_found') is True

    def test_find_group(self, args):
        data = ValueData({})

        target = Target()
        target.ip = '192.168.1.10'
        target.identifier = '192.168.1.10'

        group_type = '8'

        pipeline = Pipeline()

        targetGroups = TargetGroups(args, pipeline)
        targetGroups.put_group(data, target, group_type, None)

        target.ip = '192.168.1.100'
        target.identifier = '192.168.1.100'
        target.identifier_int = 3221291364

        group = targetGroups.find_group(data, target)

        assert group is not None

    def test_get_network_range(self, args):
        targetGroups = TargetGroups(args, Pipeline())
        begin_ip, end_ip = targetGroups._get_network_range('192.1.1.100/24')

        assert (begin_ip == 3221291265 and end_ip == 3221291518)

    def test_create_group_by_identifier(self, args):
        targetGroups = TargetGroups(args, Pipeline())

        target = Target()
        target.ip = '192.168.1.10'
        target.identifier = '192.168.1.10'

        group = targetGroups._create_group_by_identifier(target, '8', None)

        assert group.begin_int == 3221225473
        assert group.end_int == 3238002686

        group = targetGroups._create_group_by_identifier(target, '255.0.0.0', None)

        assert group.begin_int == 3221225473
        assert group.end_int == 3238002686

        group = targetGroups._create_group_by_identifier(target, 'network', '192.168.1.1/24')

        assert group.begin_int == 3232235777
        assert group.end_int == 3232236030

    def test_find_indexes(self, args):
        targetGroups = TargetGroups(args, Pipeline())

        targetGroups.group_indexes = [100, 200, 299, 310, 350, 400]
        targetGroups.groups = [None, None, None, (300, 310, '300'), None, None]

        group = targetGroups._find_indexes(300)

        assert group is not None

        targetGroups.group_indexes = [10, 200, 280, 305, 350, 400]
        targetGroups.groups = [None, None, None, (300, 310, '300'), None, None]

        group = targetGroups._find_indexes(300)

        assert group is not None

    def test_add_indexes(self, args):
        targetGroups = TargetGroups(args, Pipeline())

        group = TargetGroup(100, 300, '300')

        targetGroups._add_indexes(group)

        assert targetGroups.group_indexes[0] == 300
        assert len(targetGroups.groups) == 1



from abc import ABC

from ipscap.configs import Constant
from ipscap.util.raw_socket_entity import IPHeader
from ipsurv.util.sys_util import System
import base64
import copy


class ProtocolService(ABC):
    _instances = {}

    @classmethod
    def register_services(cls, icmp_service, tcp_service, udp_service):
        cls._instances[IPHeader.PROTOCOL_ICMP] = icmp_service
        cls._instances[IPHeader.PROTOCOL_TCP] = tcp_service
        cls._instances[IPHeader.PROTOCOL_UDP] = udp_service

    @classmethod
    def get_service(cls, protocol):
        return cls._instances.get(protocol)

    def show_transfer(self, ip_header, protocol_header, passage_num, now, args):
        if args.fixed_output != Constant.OUTPUT_NONE:
            if args.fixed_output != Constant.OUTPUT_LINE:
                self.show_overview(ip_header, protocol_header, now, passage_num, args.output_raw)
                self.show_payload(ip_header, protocol_header, args.fixed_output)
            else:
                self.show_line(ip_header, protocol_header, now, passage_num)

    def show_overview(self, ip_header, protocol_header, now, passage_num, output_raw):
        pass

    def create_decorated_ip_header(self, ip_header):
        ip_header = copy.deepcopy(ip_header)

        self.add_raw(ip_header, 'total_length')
        self.add_raw(ip_header, 'identification')
        self.add_raw(ip_header, 'ttl')
        self.add_raw(ip_header, 'src_ip')
        self.add_raw(ip_header, 'dest_ip')
        self.add_raw(ip_header, 'checksum')

        return ip_header

    def create_decorated_protocol_header(self, protocol_header):
        pass

    def create_decorated_headers(self, ip_header, protocol_header):
        ip_header = self.create_decorated_ip_header(ip_header)
        protocol_header = self.create_decorated_protocol_header(protocol_header)

        return ip_header, protocol_header

    def show_head(self, ip_header, now, passage_num):
        System.line(self.label('Time:') + self.get_datatime(now) + ' / ' + self.get_timestamp(now) + ', Passage number: ' + str(passage_num))
        System.line(self.label('IP header:') + 'Version: ' + str(ip_header.version) + ', IP header length: ' + str(ip_header.iph_length) + ', Identification: ' + str(ip_header.identification) + ', Total length: ' + str(ip_header.total_length) + ', Checksum: ' + str(ip_header.checksum) + ', TTL: ' + str(ip_header.ttl) + ', IP protocol: ' + ip_header.protocol_code + "[{}]".format(str(ip_header.protocol)))

    def show_middle(self, ip_header, protocol_header, show_data_len):
        System.line(self.label('Source:') + self.label('IP: ' + str(ip_header.src_ip), 35) + 'Port: ' + str(protocol_header.src_port))
        System.line(self.label('Destination:') + self.label('IP: ' + str(ip_header.dest_ip), 35) + 'Port: ' + str(protocol_header.dest_port))

        if ip_header.direction == 1:
            direction_msg = 'SEND [ >>> ]'
        elif ip_header.direction == 2:
            direction_msg = 'RECEIVE [ <<< ]'
        else:
            direction_msg = 'UNKNOWN'

        System.line(self.label('Direction:') + direction_msg)

        if show_data_len:
            System.line(self.label('Data length:') + str(protocol_header.payload_length) + ' byte')

    def show_line(self, ip_header, protocol_header, now, passage_num, split=', '):
        pass

    def show_payload(self, ip_header, protocol_header, output):
        if output == Constant.OUTPUT_TEXT:
            data = protocol_header.payload_data.decode('utf-8', errors='ignore')

            if len(protocol_header.get_sanitized_data()) > 0:
                System.line(data + "\n")
        elif output == Constant.OUTPUT_BINARY:
            System.line(protocol_header.payload_data)
            System.line('')
        elif output == Constant.OUTPUT_BINARY_ALL:
            System.line(self.get_all_data(ip_header, protocol_header))
            System.line('')
        elif output == Constant.OUTPUT_HEX:
            hex_data = self.get_hex_data(protocol_header.payload_data)
            System.line(hex_data + "\n")
        elif output == Constant.OUTPUT_HEX_ALL:
            hex_data = self.get_hex_data(self.get_all_data(ip_header, protocol_header))
            System.line(hex_data + "\n")
        elif output == Constant.OUTPUT_BASE64:
            data = self.get_base64_data(protocol_header.payload_data)
            System.line(data + "\n")
        elif output == Constant.OUTPUT_BASE64_ALL:
            data = self.get_base64_data(self.get_all_data(ip_header, protocol_header))
            System.line(data + "\n")

    def get_all_data(self, ip_header, protocol_header):
        return ip_header.header_data + protocol_header.header_data + protocol_header.payload_data

    def get_datatime(self, now):
        return now.strftime("%Y-%m-%d %H:%M:%S.%f")[:-2]

    def get_timestamp(self, now):
        return str(now.timestamp())[:-2]

    def get_hex_data(self, data):
        hex_data = ''.join(f'{byte:02x} ' for byte in data)

        return hex_data

    def get_base64_data(self, data):
        base64_data = base64.b64encode(data)

        return base64_data.decode('utf-8')

    def label(self, v, n=16):
        return v.ljust(n)

    def add_raw(self, header, name, raw_name=None):
        v = str(getattr(header, name))

        raw_name = raw_name if raw_name is not None else name

        setattr(header, name, v + '[' + header.raws[raw_name] + ']')


class ICMPProtocolService(ProtocolService):
    def show_overview(self, ip_header, protocol_header, now, passage_num, output_raw):
        if output_raw:
            ip_header, protocol_header = self.create_decorated_headers(ip_header, protocol_header)

        self.show_head(ip_header, now, passage_num)

        System.line(self.label('ICMP header:') + 'ICMP header length: ' + str(protocol_header.icmph_length) + ', Type: ' + str(protocol_header.icmp_type) + ', Code: ' + str(protocol_header.code))

        self.show_middle(ip_header, protocol_header, False)

        System.line(self.label('IP-H data:') + self.get_hex_data(ip_header.header_data))
        System.line(self.label('ICMP-H data:') + self.get_hex_data(protocol_header.header_data))
        System.line('')

    def create_decorated_protocol_header(self, protocol_header):
        protocol_header = copy.deepcopy(protocol_header)

        self.add_raw(protocol_header, 'icmp_type')
        self.add_raw(protocol_header, 'code')
        self.add_raw(protocol_header, 'checksum')

        return protocol_header

    def show_line(self, ip_header, protocol_header, now, passage_num, split=', '):
        line = ''

        line += self.get_datatime(now) + split
        line += str(passage_num) + split
        line = line.ljust(30)

        line += str(ip_header.version) + split + str(ip_header.iph_length) + split + str(ip_header.ttl) + split + str(ip_header.total_length) + split
        line = line.ljust(50)

        line += str(ip_header.protocol_code) + split
        line += str(protocol_header.icmph_length) + split
        line = line.ljust(60)

        line += str(protocol_header.icmp_type) + split + str(protocol_header.code) + split
        line = line.ljust(70)

        line += str(ip_header.src_ip) + split
        line = line.ljust(95)

        line += str(ip_header.dest_ip) + split
        line = line.ljust(120)

        line += ip_header.direction_code + split

        System.line(line)

    def show_payload(self, ip_header, protocol_header, output):
        pass


class TCPProtocolService(ProtocolService):
    def show_overview(self, ip_header, protocol_header, now, passage_num, output_raw):
        if output_raw:
            ip_header, protocol_header = self.create_decorated_headers(ip_header, protocol_header)

        self.show_head(ip_header, now, passage_num)

        System.line(self.label('TCP header:') + 'TCP header length: ' + str(protocol_header.tcph_length) + ', Checksum: ' + str(protocol_header.checksum) + ', Sequence: ' + str(protocol_header.seq_no) + ', Acknowledgement: ' + str(protocol_header.ack_no) + ', Window: ' + str(protocol_header.window) + ', Flags: ' + str(protocol_header.flag_codes))
        System.line(self.label('TCP options:') + self._get_tcp_options(protocol_header.tcp_options))

        self.show_middle(ip_header, protocol_header, True)

        System.line(self.label('IP-H data:') + self.get_hex_data(ip_header.header_data))
        System.line(self.label('TCP-H data:') + self.get_hex_data(protocol_header.header_data))
        System.line('')

    def create_decorated_protocol_header(self, protocol_header):
        protocol_header = copy.deepcopy(protocol_header)

        self.add_raw(protocol_header, 'src_port')
        self.add_raw(protocol_header, 'dest_port')
        self.add_raw(protocol_header, 'seq_no')
        self.add_raw(protocol_header, 'ack_no')
        self.add_raw(protocol_header, 'flags')
        self.add_raw(protocol_header, 'window')
        self.add_raw(protocol_header, 'checksum')

        return protocol_header

    def show_line(self, ip_header, protocol_header, now, passage_num, split=', '):
        line = ''

        line += self.get_datatime(now) + split
        line += str(passage_num) + split
        line = line.ljust(30)

        line += str(ip_header.version) + split + str(ip_header.iph_length) + split + str(ip_header.ttl) + split + str(ip_header.total_length) + split
        line = line.ljust(50)

        line += str(ip_header.protocol_code) + split
        line += str(protocol_header.tcph_length) + split + str(protocol_header.seq_no) + split + str(protocol_header.ack_no) + split + str(protocol_header.window) + split
        line = line.ljust(90)

        line += str(protocol_header.flag_codes) + split
        line = line.ljust(112)

        line += str(protocol_header.payload_length) + split
        line = line.ljust(120)

        line += str(ip_header.src_ip) + ':' + str(protocol_header.src_port) + split
        line = line.ljust(145)

        line += str(ip_header.dest_ip) + ':' + str(protocol_header.dest_port) + split
        line = line.ljust(170)

        line += ip_header.direction_code + split
        line = line.ljust(185)

        line += self._get_tcp_options(protocol_header.tcp_options, ';')

        System.line(line)

    def _get_tcp_options(self, tcp_options, split=', '):
        value = ''

        if len(tcp_options) > 0:
            for k, v in tcp_options.items():
                if isinstance(v, bool):
                    value += k
                else:
                    value += k + ':' + str(v)

                value += split

            value = value.strip(split)
        else:
            value = '-'

        return value


class UDPProtocolService(ProtocolService):
    def show_overview(self, ip_header, protocol_header, now, passage_num, output_raw):
        if output_raw:
            ip_header, protocol_header = self.create_decorated_headers(ip_header, protocol_header)

        self.show_head(ip_header, now, passage_num)

        System.line(self.label('UDP header:') + 'UDP header length: ' + str(protocol_header.udph_length) + ', Checksum: ' + str(protocol_header.checksum))

        self.show_middle(ip_header, protocol_header, True)

        System.line(self.label('IP-H data:') + self.get_hex_data(ip_header.header_data))
        System.line(self.label('UDP-H data:') + self.get_hex_data(protocol_header.header_data))
        System.line('')

    def create_decorated_protocol_header(self, protocol_header):
        protocol_header = copy.deepcopy(protocol_header)

        self.add_raw(protocol_header, 'src_port')
        self.add_raw(protocol_header, 'dest_port')
        self.add_raw(protocol_header, 'udph_length')

        return protocol_header

    def show_line(self, ip_header, protocol_header, now, passage_num, split=', '):
        line = ''

        line += self.get_datatime(now) + split
        line += str(passage_num) + split
        line = line.ljust(30)

        line += str(ip_header.version) + split + str(ip_header.iph_length) + split + str(ip_header.ttl) + split + str(ip_header.total_length) + split
        line = line.ljust(50)

        line += str(ip_header.protocol_code) + split
        line += str(protocol_header.udph_length) + split
        line = line.ljust(55)

        line += str(protocol_header.payload_length) + split
        line = line.ljust(70)

        line += str(ip_header.src_ip) + ':' + str(protocol_header.src_port) + split
        line = line.ljust(95)

        line += str(ip_header.dest_ip) + ':' + str(protocol_header.dest_port) + split
        line = line.ljust(120)

        line += ip_header.direction_code + split

        System.line(line)
